from __future__ import print_function, division, absolute_import

correct_manifest_output = """
+-------------------------------------------+
\tManifest
+-------------------------------------------+

General:
\tServer: access.stage.redhat.com/management/distributors/
\tServer Version: 0.7.13.13-1
\tDate Created: 2013-02-21T15:31:44.058+0000
\tCreator: stage_test_6

Consumer:
\tName: sam_org
\tUUID: ba5ac769-207e-421c-bfd2-a23c767114af
\tContent Access Mode: entitlement
\tType: sam
\tAPI URL: https://subscription.rhn.stage.redhat.com/subscription/consumers/
\tWeb URL: access.stage.redhat.com/management/distributors/

Subscription:
\tName: RHN Monitoring (Up to 1 guest)
\tQuantity: 1
\tCreated: 2013-02-21T15:31:13.000+0000
\tStart Date: 2012-12-31T05:00:00.000+0000
\tEnd Date: 2013-12-31T04:59:59.000+0000
\tService Level: Layered
\tService Type: L1-L3
\tArchitectures:%(space)s
\tSKU: RH1569626
\tContract:%(space)s
\tOrder:%(space)s
\tAccount: 5206743
\tVirt Limit:%(space)s
\tRequires Virt-who: False
\tEntitlement File: export/entitlements/8a99f9833cf86efc013cfd613be066cb.json
\tCertificate File: export/entitlement_certificates/2414805806930829936.pem
\tCertificate Version: 1.0
\tProvided Products:
\tContent Sets:

""" % ({'space': ' '})

consumer_json = """
{
    "name": "sam_org",
    "type": {
        "id": "5",
        "label": "sam",
        "manifest": true
    },
    "uuid": "ba5ac769-207e-421c-bfd2-a23c767114af",
    "urlApi": "https://subscription.rhn.stage.redhat.com/subscription/consumers/",
    "urlWeb": "access.stage.redhat.com/management/distributors/"
}"""

meta_json = """
{
    "created": "2013-02-21T15:31:44.058+0000",
    "principalName": "stage_test_6",
    "version": "0.7.13.13-1",
    "webAppPrefix": "access.stage.redhat.com/management/distributors/"
}"""

ent_cert = """-----BEGIN CERTIFICATE-----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-----END CERTIFICATE-----"""

ent_cert_private = """-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEAox/RWlLuCi+6L84tdDx4CDEN9J2S1BzuoKdW6CLbQxLMJ/YW
miNX+Z8ox9ucCjA9aLrpkB+JH4lr7/dBlyPMJP5H6gQ9kMz/Y75lvl98E9Gh/09E
bjAu2ZkTmLB+IV/xPDZVHfugRnCy7sV1ElTkklYtGpfDTZ97eIhcSiLaLG1OT2rn
1DqMDYF5YLFkk2Wr7DK619gIGMY6XW7uBzIj+xmOYn2R4m5PAnNhFRJ8MdHfjTwg
EkA9w8r/KcqydsJ6/xCZOBBZlLHq0+CF3pbR9fuN+sGBL43BU+0Kd2MqPF64JTVa
Bkz2TZta+KUAHVhDKmGDEtuWnlcenOyJXpOg5QIDAQABAoIBAQCQUxZnU/hICvIw
WbrdnKKWnNA8HS5LfU2j1mqN+EkGjxsSQCi/N4Ye1TK/oQ3t9cBfaQA9A6nOvUdC
iQD/OCzmjhQSeK3/72AGX+6lANZGsrMhsIBa/UZ2P3mXHpB59uj43Rlx7a9go1Ws
7AcosPOKhbRiUuP5SaF0gzEkZrGhm6p4IJSEvWOmupBw2z21bnjyrlg0YQ+uBPFI
xmalD7do4Yw99ef3DoCyL8iEehv/N2McPP61tsToxwpLqw2Q4ELxtbxAkDVUXu9q
4RIbNm+OUcZZB46NxW/sk6nX/9LBlRG/1S8oTO4G6f2AK98ltWxNm/0BoV/KkxpN
eCI07/ABAoGBANE/ITxTtnrRlpvpn0qvOWCr/HIS8xurTk5vTnwHIys4vvwv23p7
nc3DuaLYIeqs0sujJaBY6mZ8QnAnMO4UUozHksFx+CTs6Dx/RqeF/vJoOb+YvmcM
3QrGZD+nCUHrBOVRfWGBttvqDh1rObOEXlWzWsim4Doo27IwhtwAYevdAoGBAMeS
f92iD8Gszu46IJa1o1Rt0DtV4KrOlAoahoXewkWl/M8Xf3OZt60J9IoEIiffDDTj
KijBAsq1HT4dfa6lM6hYTQocySIgomNolw5l1f0znUO9N8QklHeahCwaw2V8aCYX
FCkhJGpEFNmfUgNWM32OuYrSIn1TEUzqVn2y4dypAoGAbDUMV+kmjb8C9p/K21Fg
B6kJBGjeRWnCNfeDi8oZGsneogWRp3ZztavIvPiuGXDEFcDJvXEdzl/l75+kwwnJ
Yrn2H4lzfIzy0A41mH5HyE2zx2wS0rGpQWA1CWG0/NyvjHMmtpzg1jrkj7wae8Yx
DnqQsQDzJcBpVG2Z3/1mphUCgYABiB9RHShPzTq9W1basUQyprEdc3hI91LtjOyR
ZHdLP43kLQL+aSSewF/PG18DvVODqGavb2PNGHzD+Ef5qizuUtcsh7IHgAafCrN2
GdP9oILJfU9LQxicnmP7Tq1HPyAxgqXV9vonkqQyU2W2vtegVBMafKhlG9kbJQVK
66+OGQKBgEBpo/BpYXNifIuxZqX4CBpBNSez4p04T1rVN4epCpE8+Lc8zZLLWd4Y
SdHfu5F7EFQ+KERyvi4v31HQkdT99+4D7PZq43DzfosKCseaGT12GxWxlVJP5MZR
PNe6469nI+tuqa+9+Vbe6eUsvAePcb7Mbc3JCzzzHJLvJsMEcJgo
-----END RSA PRIVATE KEY-----"""

entitlement_json = """
{
    "accountNumber": "5206743",
    "certificates": [
        {
            "cert":""" + ' "' + ent_cert.replace("\n", "") + '",' + """
            "created": "2013-02-21T15:31:14.000+0000",
            "id": "8a99f9833cf86efc013cfd613f2266cc",
            "key":""" + ' "' + ent_cert_private.replace("\n", "") + '",' + """
            "serial": {
                "collected": false,
                "created": "2013-02-21T15:31:14.000+0000",
                "expiration": "2013-12-31T04:59:59.000+0000",
                "id": 2414805806930829936,
                "revoked": false,
                "serial": 2414805806930829936,
                "updated": "2013-02-21T15:31:14.000+0000"
            },
            "updated": "2013-02-21T15:31:14.000+0000"
        }
    ],
    "contractNumber": null,
    "created": "2013-02-21T15:31:13.000+0000",
    "endDate": "2013-12-31T04:59:59.000+0000",
    "href": "/entitlements/8a99f9833cf86efc013cfd613be066cb",
    "id": "8a99f9833cf86efc013cfd613be066cb",
    "pool": {
        "accountNumber": "5206743",
        "activeSubscription": true,
        "attributes": [],
        "consumed": 1,
        "contractNumber": null,
        "created": "2013-01-04T02:54:05.000+0000",
        "endDate": "2013-12-31T04:59:59.000+0000",
        "exported": 1,
        "href": "/pools/8a99f9833c01cc09013c037ace480344",
        "id": "8a99f9833c01cc09013c037ace480344",
        "owner": {
            "contentPrefix": null,
            "created": "2012-10-23T05:05:02.000+0000",
            "defaultServiceLevel": null,
            "displayName": "6752568",
            "href": "/owners/6752568",
            "id": "8a99f9833a7a39f2013a8c0277dd7242",
            "key": "6752568",
            "parentOwner": null,
            "updated": "2012-10-23T05:05:02.000+0000",
            "upstreamUuid": null
        },
        "productAttributes": [
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace480345",
                "name": "support_type",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "L1-L3"
            },
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace490346",
                "name": "name",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "RHN Monitoring (Up to 1 guest)"
            },
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace490347",
                "name": "variant",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "Smart Management"
            },
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace490348",
                "name": "type",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "MKT"
            },
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace490349",
                "name": "support_level",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "Layered"
            },
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace49034a",
                "name": "product_family",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "Red Hat Applications"
            },
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace49034b",
                "name": "support_level_exempt",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "true"
            },
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace49034c",
                "name": "description",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "Red Hat Applications"
            },
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace49034e",
                "name": "option_code",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "0"
            },
            {
                "created": "2013-01-04T02:54:05.000+0000",
                "id": "8a99f9833c01cc09013c037ace49034d",
                "name": "subtype",
                "productId": "RH1569626",
                "updated": "2013-01-04T02:54:05.000+0000",
                "value": "Layered"
            }
        ],
        "productId": "RH1569626",
        "productName": "RHN Monitoring (Up to 1 guest)",
        "providedProducts": [],
        "quantity": 100,
        "restrictedToUsername": null,
        "sourceEntitlement": null,
        "startDate": "2012-12-31T05:00:00.000+0000",
        "subscriptionId": "2677371",
        "subscriptionSubKey": "master",
        "updated": "2013-02-21T15:31:13.000+0000"
    },
    "quantity": 1,
    "startDate": "2012-12-31T05:00:00.000+0000",
    "updated": "2013-02-21T15:31:13.000+0000"
}
"""



from __future__ import print_function, division, absolute_import

# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

import mock
import dbus

from rhsmlib.services import consumer
from rhsmlib.dbus import constants
from rhsmlib.dbus.objects.consumer import ConsumerDBusObject

from subscription_manager import injection as inj
from subscription_manager.identity import Identity
from test.rhsmlib_test.base import InjectionMockingTest, DBusObjectTest


class TestConsumerService(InjectionMockingTest):
    def setUp(self):
        super(TestConsumerService, self).setUp()
        self.mock_identity = mock.Mock(spec=Identity, name="Identity").return_value
        self.mock_identity.is_valid.return_value = True
        self.mock_identity.uuid = "43b30b32-86cf-459e-9310-cb4182c23c4a"

    def injection_definitions(self, *args, **kwargs):
        if args[0] == inj.IDENTITY:
            return self.mock_identity
        else:
            return None

    def test_get_consumer_uuid(self):
        """
        Test of getting UUID
        """
        test_concumer = consumer.Consumer()
        uuid = test_concumer.get_consumer_uuid()
        self.assertEqual(uuid, "43b30b32-86cf-459e-9310-cb4182c23c4a")

    def test_get_consumer_uuid_unregistered_system(self):
        """
        When system is not registered, then get_consumer_uuid should
        return empty string.
        :return:
        """
        self.mock_identity.uuid = None
        test_concumer = consumer.Consumer()
        uuid = test_concumer.get_consumer_uuid()
        self.assertEqual(uuid, "")


class TestConsumerDBusObject(DBusObjectTest, InjectionMockingTest):
    def setUp(self):
        super(TestConsumerDBusObject, self).setUp()
        self.proxy = self.proxy_for(ConsumerDBusObject.default_dbus_path)
        self.interface = dbus.Interface(self.proxy, constants.CONSUMER_INTERFACE)

        consumer_patcher = mock.patch('rhsmlib.dbus.objects.consumer.Consumer', autospec=True)
        self.mock_consumer = consumer_patcher.start().return_value
        self.addCleanup(consumer_patcher.stop)

    def _create_mock_identity(self):
        self.mock_identity = mock.Mock(spec=Identity, name="Identity").return_value
        self.mock_identity.is_valid.return_value = True
        self.mock_identity.uuid = "43b30b32-86cf-459e-9310-cb4182c23c4a"

    def injection_definitions(self, *args, **kwargs):
        if args[0] == inj.IDENTITY:
            if not hasattr(self, 'mock_identity'):
                self._create_mock_identity()
            return self.mock_identity
        else:
            return None

    def dbus_objects(self):
        return [ConsumerDBusObject]

    def test_get_consumer_uuid(self):
        """
        Test of getting consumer UUID
        """
        expected_result = "43b30b32-86cf-459e-9310-cb4182c23c4a"

        def assertions(*args):
            result = args[0]
            self.assertEqual(result, expected_result)

        self.mock_consumer.get_consumer_uuid.return_value = expected_result

        dbus_method_args = ['']
        self.dbus_request(assertions, self.interface.GetUuid, dbus_method_args)



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import

import imp
import os
import types

from nose.plugins.skip import SkipTest

try:
    import dnf
    import librepo
except ImportError as e:
    raise SkipTest(e)


from . import fixture


# Yeah, this is weird. The yum plugins aren't on sys.path, nor are they in the
# local src path that nosetest searches for modules. src/plugins is also not a
# package dir (no __init__). And to top it off, the module name isn't a valid
# python module name ('product-id.py', ie with an invalid '-').
rel_path = "../src/dnf-plugins/product-id.py"
plugin_file_path = os.path.join(os.path.dirname(__file__), rel_path)
plugin_file = open(plugin_file_path, 'r')

dir_path, module_name = os.path.split(plugin_file_path)
module_name = module_name.split(".py")[0]


# NOTE: the yum plugin 'product-id' gets imported as yum_product_id
fp, pathname, description = imp.find_module(module_name, [dir_path])
try:
    dnf_product_id = imp.load_module('dnf_product_id', fp, pathname, description)
except ImportError as e:
    raise SkipTest(e)
finally:
    fp.close()


class TestDnfPluginModule(fixture.SubManFixture):
    def setUp(self):
        super(TestDnfPluginModule, self).setUp()

    def test(self):
        self.assertTrue(isinstance(dnf_product_id, types.ModuleType))
        self.assertTrue(isinstance(dnf, types.ModuleType))
        self.assertTrue(isinstance(librepo, types.ModuleType))



from __future__ import print_function, division, absolute_import

from six.moves import configparser
from unittest import TestCase
import os
import subprocess
import tempfile

from nose.plugins.attrib import attr


@attr('zypper')
class TestServicePlugin(TestCase):
    def setUp(self):
        missing = []
        for name in ['RHSM_USER', 'RHSM_PASSWORD', 'RHSM_URL', 'RHSM_POOL', 'RHSM_TEST_REPO', 'RHSM_TEST_PACKAGE']:
            if name not in os.environ:
                missing.append(name)
        if missing:
            raise EnvironmentError('Missing {0} environment variables'.format(str(missing)))

        # start in a non-registered state
        subprocess.call('subscription-manager unregister', shell=True)

    def has_subman_repos(self):
        repos = configparser.ConfigParser()
        with tempfile.NamedTemporaryFile(suffix='.repo') as repofile:
            subprocess.call('zypper lr -e {0}'.format(repofile.name), shell=True)
            repos.read(repofile.name)
        for repo in repos.sections():
            repo_info = dict(repos.items(repo))
            service = repo_info.get('service', None)
            if service == 'rhsm':
                return True
        return False

    def test_provides_no_subman_repos_if_unregistered(self):
        self.assertFalse(self.has_subman_repos())

    def test_provides_subman_repos_if_registered_and_subscribed(self):
        subprocess.call('subscription-manager register --username={RHSM_USER} --password={RHSM_PASSWORD} --serverurl={RHSM_URL}'.format(**os.environ), shell=True)
        subprocess.call('subscription-manager attach --pool={RHSM_POOL}'.format(**os.environ), shell=True)
        self.assertTrue(self.has_subman_repos())

    def test_can_download_rpm(self):
        subprocess.check_call('subscription-manager register --username={RHSM_USER} --password={RHSM_PASSWORD} --serverurl={RHSM_URL}'.format(**os.environ), shell=True)
        subprocess.check_call('subscription-manager attach --pool={RHSM_POOL}'.format(**os.environ), shell=True)
        subprocess.check_call('subscription-manager repos --enable={RHSM_TEST_REPO}'.format(**os.environ), shell=True)

        # remove cached subman packages
        subprocess.call('rm -rf /var/cache/zypp/packages/subscription-manager*', shell=True)
        # remove test package if installed
        subprocess.call('zypper --non-interactive rm {RHSM_TEST_PACKAGE}'.format(**os.environ), shell=True)
        subprocess.call('zypper --non-interactive --no-gpg-checks in --download-only {RHSM_TEST_PACKAGE}'.format(**os.environ), shell=True)

        subprocess.check_call('test "$(find /var/cache/zypp/packages/ -name \'{RHSM_TEST_PACKAGE}*.rpm\' | wc -l)" -gt 0'.format(**os.environ), shell=True)



from __future__ import print_function, division, absolute_import

# Copyright (c) 2010-2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
#
# Compat module that implements a subprocess.check_output work-a-like for
# python 2.6.

import logging
import subprocess
import six

log = logging.getLogger(__name__)


def check_output_2_6(*args, **kwargs):
    cmd_args = kwargs.get('args', None) or args[0]

    log.debug("Running '%s'" % cmd_args)

    process = subprocess.Popen(*args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)
    (std_output, std_error) = process.communicate()

    output = std_output.strip()

    returncode = process.poll()
    if returncode:
        raise subprocess.CalledProcessError(returncode, cmd_args)

    return output


def check_output_six(*args, **kwargs):
    output = subprocess.check_output(*args, **kwargs)
    if six.PY3 and isinstance(output, bytes):
        output = output.decode('utf-8')
    return output


check_output = check_output_2_6

if hasattr(subprocess, 'check_output'):
    check_output = check_output_six



from __future__ import print_function, division, absolute_import

# Copyright (c) 2018 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

from rhsm.config import initConfig
from rhsmlib.services import config
from six.moves import configparser
import logging
import os.path
import fnmatch
import time

try:
    import pyinotify
except ImportError:
    pyinotify = None


log = logging.getLogger(__name__)
conf = config.Config(initConfig())


class FilesystemWatcher(object):
    """
    Watches a set of directories and notifies when there are changes

    Polling implementation
    Calls callbacks associated with directory when the directory changes
    Uses a loop running in its own thread
    ** Use create_filesystem_watcher to create an instance of a filesystem_watcher
    """
    def __init__(self, dir_watches):
        """
        :param dir_watches: list of directories to watch (see DirectoryWatch class below)
        """
        self.dir_watches = set(dir_watches)
        self.should_stop = False

    def stop(self):
        self.should_stop = True

    def update(self):
        """
        calls all callbacks in dir watches that have changed according to mtime
        :return: set of dir watches that have changed, for testing purposes
        """
        changed_dir_watches = self.changed_dw_set()
        for dir_watch in changed_dir_watches:
            dir_watch.notify()
        return changed_dir_watches  # returned a value for testing purposes (test_file_monitor.py)

    def get_mtime(self, dw):
        """
        :param dw: directory watch we are looking at
        :return: timestamp of directory watch we are looking at
        """
        try:
            timestamp = os.path.getmtime("%s" % dw.path)
        except OSError:
            timestamp = None
        return timestamp

    def changed_dw_set(self):
        """
        checks which dir watches out of dir watch list have changed according to mtime of path,
        updates dir watch timestamp if changed
        :return: set of changed dir watches
        """
        changed_dws = set()
        for dir_watch in self.dir_watches:
            timestamp = self.get_mtime(dir_watch)
            if dir_watch.timestamp != timestamp:
                changed_dws.add(dir_watch)
                dir_watch.timestamp = timestamp
        return changed_dws

    def loop(self, user_end_loop_cb=None):
        """
        Loops while self.should_stop is false and the callback() is not true

        Initializes timestamps for each dir watch in dir watch list, sets callback,
        notifies dir watch if it has changed
        :param user_end_loop_cb: callback function to be called at end of each iteration of the loop
        :return:
        """
        for dw in self.dir_watches:
            dw.timestamp = self.get_mtime(dw)
        while not end_loop_cb(self, user_callback=user_end_loop_cb):
            self.update()
            time.sleep(2.0)
        self.stop()


class InotifyFilesystemWatcher(FilesystemWatcher):
    """
    Watches a set of directories and notifies when there are changes

    Inotify implementation
    Calls callbacks associated with directory when the directory changes
    Uses a loop running in its own thread
    ** Use create_filesystem_watcher to create instance of filesystem watcher
    """
    def __init__(self, dir_watches):
        """
        Filesystem watcher if pyinotify is configured and available
        loop function will override parent class loop function
        :param dir_watches: list of directories to watch (see DirectoryWatch class below)
        """
        super(InotifyFilesystemWatcher, self).__init__(dir_watches)
        self.watch_manager = None
        self.notifier = None

    def loop(self, callback=None):
        """
        sets up watch manager, notifier, adds watches to watch manager, and starts loop
        :param callback: callback method to be called at the end of each iteration of the loop
        :return:
        """
        self.watch_manager = pyinotify.WatchManager()
        self.notifier = pyinotify.Notifier(self.watch_manager, self.handle_event)
        self.add_watches()

        def inotify_callback():
            return end_loop_cb(self, user_callback=callback)

        while not inotify_callback():
            self.notifier.process_events()
            if self.notifier.check_events(500):  # keeps tests reasonably fast while still timing out
                self.notifier.read_events()

    def handle_event(self, event):
        """
        default process function for pyinotify notifier
        :param event: pyinotify Event object, has path and mask of flags representing file modification
        :return:
        """
        for dir_watch in self.dir_watches:
            if dir_watch.paths_match(event.path, event.pathname) and dir_watch.file_modified(event.mask):
                dir_watch.notify()

    def add_watches(self):
        """
        adds watches to the watch manager
        :return:
        """
        for dir_watch in self.dir_watches:
            if dir_watch.is_file:
                # watch for any changes in the directory, but only be notified of the specific path
                dir_name = os.path.abspath(os.path.dirname(dir_watch.path))
                self.watch_manager.add_watch(dir_name, dir_watch.mask, self.handle_event, do_glob=dir_watch.is_glob)
            else:
                # is already directory
                self.watch_manager.add_watch(dir_watch.path, dir_watch.mask, self.handle_event, do_glob=dir_watch.is_glob)


class DirectoryWatch(object):
    """
    Directory to be watched

    Included in list to be passed into filesystem watcher object
    Example usage:
        directory_watch = DirectoryWatch("~/home/", [my_function1, my_function2])
    """
    def __init__(self, path, callbacks, is_glob=False):
        """
        :param path: path associated with directory to be watched
        :param callbacks: list of methods called when directory is changed
        :param is_glob: bool - if path provided is glob or not
        """
        self.IN_DELETE = 0x00000200
        self.IN_MODIFY = 0x00000002
        self.IN_MOVED_TO = 0x00000080

        self.path = os.path.abspath(path)
        self.is_file = not os.path.isdir(self.path)  # used isdir because if path does not exist, assumed to be file
        self.timestamp = None
        self.is_glob = is_glob
        self.callbacks = callbacks
        self.mask = self.IN_DELETE | self.IN_MODIFY | self.IN_MOVED_TO

    def notify(self):
        """
        calls all callbacks associated with dir watch
        :return:
        """
        for cb in self.callbacks:
            if cb is not None:
                try:
                    cb()
                except Exception as e:
                    log.exception(e)

    def paths_match(self, event_path, event_pathname):
        """
        checks if event path matches any of the dir watch paths associated to it
        :param event_path: path of the inotify event object
        :param event_pathname: pathname of the inotify event object
        see pyinotify event class for more info
        :return: bool - if paths match or not
        """
        event_pathname = os.path.realpath(event_pathname)
        event_path = os.path.realpath(event_path)
        if self.is_file:
            # we do not care about events on other paths, like .swp files
            return fnmatch.fnmatchcase(event_pathname, self.path)
        return fnmatch.fnmatchcase(event_path, self.path)

    def file_modified(self, event_mask):
        """
        checks if any flag has been set by event corresponding to modification
        :param event_mask: mask of the inotify event object, signifies what happened
        :return: bool - if any of the modification flags have been set
        """
        return bool(self.mask & event_mask)


def end_loop_cb(fsw, user_callback=None):
    if user_callback is not None:
        try:
            return user_callback() or fsw.should_stop
        except Exception as e:
            log.exception(e)
    return fsw.should_stop


def create_filesystem_watcher(dir_watches):
    """
    determines if inotify is available and configured in rhsm.conf
    If yes, uses pyinotify. Else, uses polling methods.
    Uses inotify by default
    :param dir_watches: list of directories to watch to create
    correct filesystem watcher object
    :return: correct filesystem watcher object

    Example usage:
        filesystem_watcher = create_filesystem_watcher([directory_watch1, directory_watch2])
        thread = threading.Thread(target=filesystem_watcher.loop)
        thread.start()
    """
    available = is_inotify_available()
    configed = is_inotify_config()
    if not (available and configed):
        return FilesystemWatcher(dir_watches)
    else:
        return InotifyFilesystemWatcher(dir_watches)


def is_inotify_available():
    return pyinotify is not None


def is_inotify_config():
    """
    Check if inotify is enabled or disabled in rhsm.conf.
    It is enabled by default.
    :return: It returns True, when inotify is enabled. Otherwise it returns False.
    """
    try:
        use_inotify = conf['rhsm'].get_int('inotify')
    except ValueError as e:
        log.exception(e)
        return True
    except configparser.Error as e:
        log.exception(e)
        return True
    else:
        if use_inotify is None:
            return True

    return bool(use_inotify)



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import errno
import mock
import os
import shutil
import six
import tempfile
import zipfile
from zipfile import ZipFile

from subscription_manager.i18n_optparse import OptionParser

from . import manifestdata
from rct.manifest_commands import CatManifestCommand
from rct.manifest_commands import DumpManifestCommand
from rct.manifest_commands import get_value
from rct.manifest_commands import RCTManifestCommand
from rct.manifest_commands import ZipExtractAll

from .fixture import Capture, SubManFixture


def _build_valid_manifest():
    manifest_zip = six.BytesIO()
    manifest_object = ZipFile(manifest_zip, "w", compression=zipfile.ZIP_STORED)
    manifest_object.writestr("signature", "dummy")
    consumer_export_zip = six.BytesIO()
    consumer_export_object = ZipFile(consumer_export_zip, "w", compression=zipfile.ZIP_STORED)
    consumer_export_object.writestr("export/consumer.json", manifestdata.consumer_json)
    consumer_export_object.writestr("export/meta.json", manifestdata.meta_json)
    consumer_export_object.writestr("export/entitlements/8a99f9833cf86efc013cfd613be066cb.json",
            manifestdata.entitlement_json)
    consumer_export_object.writestr("export/entitlement_certificates/2414805806930829936.pem",
            manifestdata.ent_cert + '\n' + manifestdata.ent_cert_private)
    consumer_export_object.close()
    manifest_object.writestr("consumer_export.zip", consumer_export_zip.getvalue())
    manifest_object.close()
    return manifest_zip


class RCTManifestCommandTests(SubManFixture):

    def test_get_value(self):
        data = {"test": "value", "test2": {"key2": "value2", "key3": []}}
        self.assertEqual("", get_value(data, "some.test"))
        self.assertEqual("", get_value(data, ""))
        self.assertEqual("", get_value(data, "test2.key4"))
        self.assertEqual("", get_value(data, "test2.key2.fred"))
        self.assertEqual("value", get_value(data, "test"))
        self.assertEqual("value2", get_value(data, "test2.key2"))
        self.assertEqual([], get_value(data, "test2.key3"))

    def test_cat_manifest(self):
        catman = CatManifestCommand()
        parser = OptionParser()
        parser.add_option("--no-content")
        (options, args) = parser.parse_args([])
        catman.options = options
        catman.args = [_build_valid_manifest()]

        with Capture() as cap:
            catman._do_command()

        self.assertEqual("", cap.err)
        self.assert_string_equals(manifestdata.correct_manifest_output, cap.out)

    def test_extract_manifest(self):
        tmp_dir = tempfile.mkdtemp()
        mancommand = RCTManifestCommand()
        mancommand.args = [_build_valid_manifest()]
        mancommand._extract_manifest(tmp_dir)

        self.assertTrue(os.path.exists(os.path.join(tmp_dir, "export")))

        shutil.rmtree(tmp_dir)

    def test_dump_manifest_current(self):
        original_directory = os.getcwd()
        new_directory = tempfile.mkdtemp()
        os.chdir(new_directory)
        mancommand = DumpManifestCommand()
        mancommand.args = [_build_valid_manifest()]

        #This makes sure there is a 'None' at 'self.options.destination'
        mancommand.options = mancommand
        mancommand.destination = None
        mancommand.overwrite_files = False

        mancommand._do_command()
        self.assertTrue(os.path.exists(os.path.join(new_directory, "export")))
        os.chdir(original_directory)
        shutil.rmtree(new_directory)

    def test_dump_manifest_directory(self):
        new_directory = tempfile.mkdtemp()
        mancommand = DumpManifestCommand()
        mancommand.args = [_build_valid_manifest()]

        #This makes sure the temp directory is referenced at 'self.options.destination'
        mancommand.options = mancommand
        mancommand.destination = new_directory
        mancommand.overwrite_files = False

        mancommand._do_command()
        self.assertTrue(os.path.exists(os.path.join(new_directory, "export")))
        shutil.rmtree(new_directory)

    def test_dump_manifest_directory_twice(self):
        new_directory = tempfile.mkdtemp()
        mancommand = DumpManifestCommand()
        mancommand.args = [_build_valid_manifest()]

        #This makes sure the temp directory is referenced at 'self.options.destination'
        mancommand.options = mancommand
        mancommand.destination = new_directory
        mancommand.overwrite_files = True

        mancommand._do_command()
        mancommand._do_command()
        self.assertTrue(os.path.exists(os.path.join(new_directory, "export")))
        shutil.rmtree(new_directory)

    @mock.patch("rct.manifest_commands.ZipExtractAll._write_file")
    def test_dump_manifest_directory_no_perms(self, mock_write_file):
        new_directory = tempfile.mkdtemp()
        mancommand = DumpManifestCommand()
        mancommand.args = [_build_valid_manifest()]

        #This makes sure the temp directory is referenced at 'self.options.destination'
        mancommand.options = mancommand
        mancommand.destination = new_directory
        mancommand.overwrite_files = True

        mock_write_file.side_effect = IOError(errno.EACCES, "permission denied", new_directory)
        mancommand._do_command()
        # we fail to extract manifest in this case
        self.assertFalse(os.path.exists(os.path.join(new_directory, "export")))

    @mock.patch("rct.manifest_commands.ZipExtractAll._write_file")
    def test_dump_manifest_directory_exists(self, mock_write_file):
        new_directory = tempfile.mkdtemp()

        mancommand = DumpManifestCommand()
        mancommand.args = [_build_valid_manifest()]

        #This makes sure the temp directory is referenced at 'self.options.destination'
        mancommand.options = mancommand
        mancommand.destination = new_directory
        mancommand.overwrite_files = True

        mock_write_file.side_effect = OSError(errno.EEXIST, "file exists", new_directory)
        mancommand._do_command()
        # we fail to extract manifest in this case
        self.assertFalse(os.path.exists(os.path.join(new_directory, "export")))


class RCTManifestExtractTests(unittest.TestCase):

    def test_extractall_outside_base(self):
        zip_file_object = six.BytesIO()
        archive = ZipExtractAll(zip_file_object, "w", compression=zipfile.ZIP_STORED)
        archive.writestr("../../../../wat", "this is weird")

        tmp_dir = tempfile.mkdtemp()
        self.assertRaises(Exception, archive.extractall, (tmp_dir))
        archive.close()
        shutil.rmtree(tmp_dir)

    def test_extractall_net_path(self):
        zip_file_object = six.BytesIO()
        archive = ZipExtractAll(zip_file_object, "w", compression=zipfile.ZIP_STORED)
        archive.writestr(r"\\nethost\share\whatever", "this is weird")

        archive.close()
        archive = ZipExtractAll(zip_file_object, "r", compression=zipfile.ZIP_STORED)

        tmp_dir = tempfile.mkdtemp()
        archive.extractall(tmp_dir)
        archive.close()

        self.assertTrue(os.path.exists(os.path.join(tmp_dir, "\\\\nethost\\share\\whatever")))

        shutil.rmtree(tmp_dir)

    def test_extractall_local(self):
        zip_file_object = six.BytesIO()
        archive = ZipExtractAll(zip_file_object, "w", compression=zipfile.ZIP_STORED)
        archive.writestr("./some/path", "this is okay I think, though odd")

        archive.close()
        archive = ZipExtractAll(zip_file_object, "r", compression=zipfile.ZIP_STORED)

        tmp_dir = tempfile.mkdtemp()
        archive.extractall(tmp_dir)
        self.assertTrue(os.path.exists(os.path.join(tmp_dir, "./some/path")))
        archive.close()
        shutil.rmtree(tmp_dir)

    @mock.patch("sys.exit")
    def test_extractall_nonzip(self, mock_exit):
        not_zip_file_object = six.BytesIO()
        ZipExtractAll(not_zip_file_object, "r")
        mock_exit.assert_called_with(1)



from __future__ import print_function, division, absolute_import

#
# Subscription manager command line utility. This script is a modified version of
# cp_client.py from candlepin scripts
#
# Copyright (c) 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import logging
import socket
import six

import six.moves.http_client
from rhsm.https import ssl
from rhsm.connection import NoValidEntitlement

import rhsm.config

from subscription_manager import injection as inj
from subscription_manager import listing
from subscription_manager import rhelproduct
from subscription_manager.i18n import ugettext as _

log = logging.getLogger(__name__)

cfg = rhsm.config.initConfig()


class MultipleReleaseProductsError(ValueError):
    def __init__(self, certificates):
        self.certificates = certificates
        self.certificate_paths = ", ".join([certificate.path for certificate in certificates])
        super(ValueError, self).__init__(("More than one release product certificate installed. Certificate paths: %s"
                                          % self.certificate_paths))

    def translated_message(self):
        return (_("Error: More than one release product certificate installed. Certificate paths: %s")
                % ", ".join([certificate.path for certificate in self.certificates]))


class ContentConnectionProvider(object):
    def __init__(self):
        pass


class ReleaseBackend(object):

    def get_releases(self):
        provider = self._get_release_version_provider()
        return provider.get_releases()

    def _get_release_version_provider(self):
        release_provider = ApiReleaseVersionProvider()
        if release_provider.api_supported():
            return release_provider
        return CdnReleaseVersionProvider()


class ApiReleaseVersionProvider(object):

    def __init__(self):
        self.cp_provider = inj.require(inj.CP_PROVIDER)
        self.identity = inj.require(inj.IDENTITY)

    def api_supported(self):
        return self._conn().supports_resource("available_releases")

    def get_releases(self):
        return self._conn().getAvailableReleases(self.identity.uuid)

    def _conn(self):
        return self.cp_provider.get_consumer_auth_cp()


class CdnReleaseVersionProvider(object):

    def __init__(self):
        self.entitlement_dir = inj.require(inj.ENT_DIR)
        self.product_dir = inj.require(inj.PROD_DIR)
        self.cp_provider = inj.require(inj.CP_PROVIDER)
        self.content_connection = self.cp_provider.get_content_connection()

    def get_releases(self):
        # cdn base url

        # Find the rhel products
        release_products = []
        certificates = set()
        installed_products = self.product_dir.get_installed_products()
        for product_hash in installed_products:
            product_cert = installed_products[product_hash]
            products = product_cert.products
            for product in products:
                rhel_matcher = rhelproduct.RHELProductMatcher(product)
                if rhel_matcher.is_rhel():
                    release_products.append(product)
                    certificates.add(product_cert)

        if len(release_products) == 0:
            log.debug("No products with RHEL product tags found")
            return []
        elif len(release_products) > 1:
            raise MultipleReleaseProductsError(certificates=certificates)

        # Note: only release_products with one item can pass previous if-elif
        release_product = release_products[0]
        entitlements = self.entitlement_dir.list_for_product(release_product.id)

        # When there is SCA entitlement certificate, then add this cert to
        # the list too. See: https://bugzilla.redhat.com/show_bug.cgi?id=1924921
        sca_entitlements = self.entitlement_dir.list_with_sca_mode()
        entitlements.extend(sca_entitlements)

        listings = []
        ent_cert_key_pairs = set()
        for entitlement in entitlements:
            contents = entitlement.content
            for content in contents:
                # ignore content that is not enabled
                # see bz #820639
                if not content.enabled:
                    continue
                if self._is_correct_rhel(release_product.provided_tags,
                                         content.required_tags):
                    listing_path = self._build_listing_path(content.url)
                    listings.append(listing_path)
                    ent_cert_key_pairs.add((entitlement.path, entitlement.key_path()))

        # FIXME: not sure how to get the "base" content if we have multiple
        # entitlements for a product

        # for a entitlement, grant the corresponding entitlement cert
        # use it for this connection

        # hmm. We are really only supposed to have one product
        # with one content with one listing file. We shall see.
        releases = []
        listings = sorted(set(listings))
        for listing_path in listings:
            try:
                data = self.content_connection.get_versions(
                    path=listing_path,
                    ent_cert_key_pairs=ent_cert_key_pairs
                )
            except (socket.error,
                    six.moves.http_client.HTTPException,
                    ssl.SSLError,
                    NoValidEntitlement) as e:
                # content connection doesn't handle any exceptions
                # and the code that invokes this doesn't either, so
                # swallow them here.
                log.exception(e)
                continue

            # any non 200 response on fetching the release version
            # listing file returns a None here
            if not data:
                continue

            ver_listing = listing.ListingFile(data=str(data))

            # ver_listing.releases can be empty
            releases = releases + ver_listing.get_releases()

        releases_set = sorted(set(releases))
        return releases_set

    def _build_listing_path(self, content_url):
        listing_parts = content_url.split('$releasever', 1)
        listing_base = listing_parts[0]
        # FIXME: cleanup paths ("//"'s, etc)
        return u"%s/listing" % listing_base  # FIXME(khowell): ensure that my changes here don't break earlier fix

    # require tags provided by installed products?

    def _is_correct_rhel(self, product_tags, content_tags):
        # easy to pass a string instead of a list
        assert not isinstance(product_tags, six.string_types)
        assert not isinstance(content_tags, six.string_types)

        for product_tag in product_tags:
            # we are comparing the lists to see if they
            # have a matching rhel-#
            product_split = product_tag.split('-', 2)
            if product_split[0] == "rhel":
                # look for match in content tags
                for content_tag in content_tags:
                    content_split = content_tag.split('-', 2)

                    # ignore non rhel content tags
                    if content_split[0] != "rhel":
                        continue

                    # exact match
                    if product_tag == content_tag:
                        return True

                    # is this content for a base of this variant
                    if product_tag.startswith(content_tag):
                        return True
                    # else, we don't match, keep looking

        log.debug("Ignoring content with tags [%s] because it does not match installed product tags [%s]" % (
            ','.join(content_tags),
            ','.join(product_tags)
        ))
        return False



#!/usr/bin/python

from __future__ import print_function, division, absolute_import

import polib
import sys


class PotFile(object):
    def __init__(self):

        self.msgids = []
        self.po = polib.pofile("po/keys.pot")
        for entry in self.po:
            self.msgids.append(entry.msgid)

        self.msgids.sort()

po = polib.POFile()
po.metadata = {
    'Project-Id-Version': '1.0',
    'Report-Msgid-Bugs-To': 'alikins@redhat.com',
    'POT-Creation-Date': '2007-10-18 14:00+0100',
    'PO-Revision-Date': '2007-10-18 14:00+0100',
    'Last-Translator': 'gen_test_en_po.py <nothing@example.com>',
    'Language-Team': 'English <yourteam@example.com>',
    'MIME-Version': '1.0',
    'Content-Type': 'text/plain; charset=utf-8',
    'Content-Transfer-Encoding': '8bit',
}


def gen_msg(msgid, tall=False, longstring=False, wrap=False):
    msg = msgid
    if longstring:
        msg = "%s________________" % msg
    if tall:
        msg = msg + "\n" * 40
    if wrap:
        msg = "xx{%s}" % msg
    return msg


def main():
    args = sys.argv[1:]

    tall = False
    longstring = False
    wrap = False
    if "--tall" in args:
        tall = True
    if "--long" in args:
        longstring = True
    if "--wrap" in args:
        wrap = True

    potfile = PotFile()
    for pot_entry in potfile.po:
        entry = polib.POEntry(
            msgid=pot_entry.msgid,
            msgstr=gen_msg(pot_entry.msgid, tall, longstring, wrap),
            occurrences=pot_entry.occurrences
            )
        po.append(entry)

    po.save("po/en_US.po")

if __name__ == "__main__":
    main()



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging
import os
import threading
#from gi.repository import GObject
import socket

import rhsm.config
import rhsm.connection as connection
import rhsm.utils
from rhsm.utils import remove_scheme
from rhsm.utils import parse_url

from subscription_manager.ga import GObject as ga_GObject
from subscription_manager.gui.utils import show_error_window
import subscription_manager.injection as inj

from subscription_manager.gui import progress
from subscription_manager.gui import widgets

from subscription_manager.i18n import ugettext as _

log = logging.getLogger(__name__)

DIR = os.path.dirname(__file__)


# From old smolt code... Force glibc to call res_init()
# to rest the resolv configuration, including reloading
# resolv.conf. This attempt to handle the case where we
# start up with no networking, fail name resolution calls,
# and cache them for the life of the process, even after
# the network starts up, and for dhcp, updates resolv.conf
def reset_resolver():
    """Attempt to reset the system hostname resolver.
    returns 0 on success, or -1 if an error occurs."""
    try:
        import ctypes
        try:
            resolv = ctypes.CDLL("libc.so.6")
            r = resolv.__res_init()
        except (OSError, AttributeError):
            log.warn("could not find __res_init in libc.so.6")
            r = -1
        return r
    except ImportError:
        # If ctypes isn't supported (older versions of python for example)
        # Then just don't do anything
        pass
    except Exception as e:
        log.warning("reset_resolver failed: %s", e)
        pass


class NetworkConfigDialog(widgets.SubmanBaseWidget):
    """This is the dialog that allows setting http proxy settings.

    It uses the instant apply paradigm or whatever you wanna call it that the
    gnome HIG recommends. Whenever a toggle button is flipped or a text entry
    changed, the new setting will be saved.

    """
    widget_names = ["networkConfigDialog", "enableProxyButton", "enableProxyAuthButton",
                    "proxyEntry", "proxyUserEntry", "proxyPasswordEntry",
                    "cancelButton", "saveButton", "testConnectionButton",
                    "connectionStatusLabel", "enableProxyBypassButton", "noProxyEntry"]
    gui_file = "networkConfig"

    def __init__(self):
        # Get widgets we'll need to access

        super(NetworkConfigDialog, self).__init__()
        self.org_timeout = socket.getdefaulttimeout()
        self.progress_bar = None

        self.cfg = rhsm.config.initConfig()
        self.cp_provider = inj.require(inj.CP_PROVIDER)

        # Need to load values before connecting signals because when the dialog
        # starts up it seems to trigger the signals which overwrites the config
        # with the blank values.
        self.set_initial_values()
        self.enableProxyButton.connect("toggled", self.enable_action)
        self.enableProxyAuthButton.connect("toggled", self.enable_action)
        self.enableProxyBypassButton.connect("toggled", self.enable_action)

        self.enableProxyButton.connect("toggled", self.clear_connection_label)
        self.enableProxyAuthButton.connect("toggled", self.clear_connection_label)
        self.enableProxyBypassButton.connect("toggled", self.clear_connection_label)

        self.enableProxyButton.connect("toggled", self.enable_or_disable_test_button)
        self.enableProxyAuthButton.connect("toggled", self.enable_or_disable_test_button)
        self.enableProxyBypassButton.connect("toggled", self.enable_or_disable_test_button)

        self.proxyEntry.connect("changed", self.clear_connection_label)
        self.proxyUserEntry.connect("changed", self.clear_connection_label)
        self.proxyPasswordEntry.connect("changed", self.clear_connection_label)
        self.noProxyEntry.connect("changed", self.clear_connection_label)

        self.proxyEntry.connect("changed", self.enable_or_disable_test_button)
        self.proxyUserEntry.connect("changed", self.enable_or_disable_test_button)
        self.proxyPasswordEntry.connect("changed", self.enable_or_disable_test_button)
        self.noProxyEntry.connect("changed", self.enable_or_disable_test_button)

        self.proxyEntry.connect("focus-out-event", self.clean_proxy_entry)

        self.cancelButton.connect("clicked", self.on_cancel_clicked)
        self.saveButton.connect("clicked", self.on_save_clicked)
        self.testConnectionButton.connect("clicked", self.on_test_connection_clicked)

        self.networkConfigDialog.connect("delete-event", self.deleted)

    def set_initial_values(self):
        proxy_url = self.cfg.get("server", "proxy_hostname") or ""
        # append port unless not specified, then append the default of 3128
        if proxy_url:
            proxy_url = proxy_url + ':' + (self.cfg.get("server", "proxy_port") or rhsm.config.DEFAULT_PROXY_PORT)

        self.proxyEntry.set_text("%s" % proxy_url)

        # show proxy/proxy auth sections as being enabled if we have values set
        # rhn actually has a separate for config flag for enabling, which seems overkill
        if self.cfg.get("server", "proxy_hostname"):
            self.enableProxyButton.set_active(True)
        if self.cfg.get("server", "proxy_hostname") and self.cfg.get("server", "proxy_user"):
            self.enableProxyAuthButton.set_active(True)
        if self.cfg.get("server", "no_proxy"):
            no_proxy = self.cfg.get("server", "no_proxy")
            self.noProxyEntry.set_text(no_proxy)
            self.enableProxyBypassButton.set_active(True)

        self.enable_action(self.enableProxyAuthButton)
        self.enable_action(self.enableProxyButton)
        self.enable_action(self.enableProxyBypassButton)

        # the extra or "" are to make sure we don't str None
        self.proxyUserEntry.set_text(str(self.cfg.get("server", "proxy_user") or ""))
        self.proxyPasswordEntry.set_text(str(self.cfg.get("server", "proxy_password") or ""))
        self.connectionStatusLabel.set_label("")
        # If there is missing proxy information, disable the proxy test
        # button.
        self.enable_or_disable_test_button()
        if not self.enableProxyButton.get_active():
            self.enableProxyAuthButton.set_sensitive(False)
            self.enableProxyBypassButton.set_sensitive(False)

    def write_values(self, widget=None, dummy=None):
        proxy = self.proxyEntry.get_text() or ""

        # don't save these values if they are disabled in the gui

        # settings of HTTP Proxy
        if proxy and self.enableProxyButton.get_active():
            # Remove any URI scheme provided
            proxy = remove_scheme(proxy)
            # Update the proxy entry field to show we removed any scheme
            self.proxyEntry.set_text(proxy)
            try:
                proxy_hostname, proxy_port = proxy.split(':')
                self.cfg.set("server", "proxy_hostname", proxy_hostname)
                self.cfg.set("server", "proxy_port", proxy_port)
            except ValueError:
                # no port? just write out the hostname and assume default
                self.cfg.set("server", "proxy_hostname", proxy)
                self.cfg.set("server", "proxy_port", rhsm.config.DEFAULT_PROXY_PORT)
        else:
            # delete config options if we disable it in the ui
            self.cfg.set("server", "proxy_hostname", "")
            self.cfg.set("server", "proxy_port", "")

        # settings of HTTP proxy authentication
        if self.enableProxyAuthButton.get_active():
            if self.proxyUserEntry.get_text() is not None:
                self.cfg.set("server", "proxy_user",
                             str(self.proxyUserEntry.get_text()))

            if self.proxyPasswordEntry.get_text() is not None:
                self.cfg.set("server", "proxy_password",
                             str(self.proxyPasswordEntry.get_text()))
        else:
            self.cfg.set("server", "proxy_user", "")
            self.cfg.set("server", "proxy_password", "")

        # settings of bypass the HTTP proxy for specific host/domain
        if self.enableProxyBypassButton.get_active():
            if self.noProxyEntry.get_text() is not None:
                self.cfg.set("server", "no_proxy",
                         str(self.noProxyEntry.get_text()))
        else:
            self.cfg.set("server", "no_proxy", "")

        try:
            self.cfg.save()
            self.cp_provider.set_connection_info()
        except Exception:
            show_error_window(_("There was an error saving your configuration.") +
                              _("Make sure that you own %s.") % self.cfg.fileName,
                                parent=self.networkConfigDialog)

    def show(self):
        self.set_initial_values()
        self.networkConfigDialog.present()

    def on_save_clicked(self, button):
        self.write_values()
        self.networkConfigDialog.hide()

    def on_cancel_clicked(self, button):
        self.networkConfigDialog.hide()

    def _has_complete_proxy_info(self):
        return self.enableProxyButton.get_active() and self.proxyEntry.get_text().strip() and (
            not self.enableProxyAuthButton.get_active() or (
                # https connection doesn't use auth unless both are provided as non-empty
                # FIXME we should account for the above in other places, such as write_values and documentation
                self.proxyUserEntry.get_text() and self.proxyPasswordEntry.get_text()
            )
        )

    def enable_or_disable_test_button(self, widget=None):
        if self._has_complete_proxy_info():
            self.testConnectionButton.set_sensitive(True)
        else:
            self.testConnectionButton.set_sensitive(False)

    def clear_connection_label(self, entry):
        self.connectionStatusLabel.set_label("")

        # only used as callback from test_connection thread
    def on_test_connection_finish(self, result):
        if result:
            self.connectionStatusLabel.set_label(_("Proxy connection succeeded"))
        else:
            self.connectionStatusLabel.set_label(_("Proxy connection failed"))
        self._clear_progress_bar()

    def _reset_socket_timeout(self):
        socket.setdefaulttimeout(self.org_timeout)

    def test_connection_wrapper(self, proxy_host, proxy_port, proxy_user, proxy_password, no_proxy):
        connection_status = self.test_connection(proxy_host, proxy_port, proxy_user, proxy_password, no_proxy)
        ga_GObject.idle_add(self.on_test_connection_finish, connection_status)

    def test_connection(self, proxy_host, proxy_port, proxy_user, proxy_password, no_proxy):
        cp = connection.UEPConnection(
                    proxy_hostname=proxy_host,
                    proxy_port=proxy_port,
                    proxy_user=proxy_user,
                    proxy_password=proxy_password,
                    no_proxy=no_proxy)
        try:
            socket.setdefaulttimeout(10)
            cp.getStatus()

        # Either connection.RemoteServerException or connection.RestLibExecption are considered
        # acceptable exceptions because they are only thrown as a response from the server. Meaning the
        # connection through the proxy was successful.
        except (connection.RemoteServerException,
                connection.RestlibException) as e:
            log.warn("Reporting proxy connection as good despite %s" %
             e)
            return True
        except connection.NetworkException as e:
            log.warn("%s when attempting to connect through %s:%s" %
             (e.code, proxy_host, proxy_port))
            return False
        except Exception as e:
            log.exception("'%s' when attempting to connect through %s:%s" %
                      (e, proxy_host, proxy_port))
            return False
        else:
            return True
        finally:
            self._reset_socket_timeout()

    # Pass through of the return values of parse_proxy_entry
    # This was done to simplify on_test_connection_clicked
    def clean_proxy_entry(self, widget=None, dummy=None):
        proxy_url = self.proxyEntry.get_text()
        proxy_host, proxy_port = self.parse_proxy_entry(proxy_url)
        if proxy_host:
            cleaned_proxy_url = proxy_host
            if proxy_port:
                cleaned_proxy_url = cleaned_proxy_url + ":" + proxy_port
            self.proxyEntry.set_text(cleaned_proxy_url)
        return (proxy_host, proxy_port)

    def parse_proxy_entry(self, proxy_url):
        proxy_url = remove_scheme(proxy_url)
        proxy_host = None
        proxy_port = None
        try:
            proxy_info = parse_url(proxy_url, default_port=rhsm.config.DEFAULT_PROXY_PORT)
            proxy_host = proxy_info[2]
            proxy_port = proxy_info[3]

        except rhsm.utils.ServerUrlParseErrorPort as e:
            proxy_host = proxy_url.split(':')[0]
            proxy_port = rhsm.config.DEFAULT_PROXY_PORT
        except rhsm.utils.ServerUrlParseError as e:
            log.error(e)
        return (proxy_host, proxy_port)

    def on_test_connection_clicked(self, button):
        proxy_host, proxy_port = self.clean_proxy_entry()

        # ensure that we only use those values for testing if required
        # this catches the case where there was previously a user and pass in the config
        # and the user unchecks the box, leaving behind the values for the time being.
        # Alternatively we could clear those boxes when the box is unchecked
        if self.enableProxyAuthButton.get_active():
            proxy_user = self.proxyUserEntry.get_text()
            proxy_password = self.proxyPasswordEntry.get_text()
        else:
            proxy_user = None
            proxy_password = None

        no_proxy = None
        if self.enableProxyBypassButton.get_active():
            no_proxy = self.noProxyEntry.get_text()

        self._display_progress_bar()
        threading.Thread(target=self.test_connection_wrapper,
                         args=(proxy_host, proxy_port, proxy_user, proxy_password, no_proxy),
                         name='TestNetworkConnectionThread').start()

    def deleted(self, event, data):
        self.write_values()
        self.networkConfigDialog.hide()
        self._clear_progress_bar()
        return True

    def _display_progress_bar(self):
        if self.progress_bar:
            self.progress_bar.set_title(_("Testing Connection"))
            self.progress_bar.set_label(_("Please wait"))
        else:
            self.progress_bar = progress.Progress(_("Testing Connection"), _("Please wait"))
            self.timer = ga_GObject.timeout_add(100, self.progress_bar.pulse)
            self.progress_bar.set_transient_for(self.networkConfigDialog)

    def _clear_progress_bar(self):
        if not self.progress_bar:  # progress bar could be none iff self.test_connection is called directly
            return

        self.progress_bar.hide()
        ga_GObject.source_remove(self.timer)
        self.timer = 0
        self.progress_bar = None

    def enable_action(self, button):
        if button.get_name() == "enableProxyButton":
            self.proxyEntry.set_sensitive(button.get_active())
            self.proxyEntry.grab_focus()
            self.enableProxyAuthButton.set_sensitive(button.get_active())
            self.enableProxyBypassButton.set_sensitive(button.get_active())
            # Proxy authentication should only be active if proxy is also enabled
            self.proxyUserEntry.set_sensitive(button.get_active() and
                    self.enableProxyAuthButton.get_active())
            self.proxyPasswordEntry.set_sensitive(button.get_active() and
                    self.enableProxyAuthButton.get_active())
        elif button.get_name() == "enableProxyAuthButton":
            self.proxyUserEntry.set_sensitive(button.get_active())
            self.proxyPasswordEntry.set_sensitive(button.get_active())
            self.get_object("usernameLabel").set_sensitive(button.get_active())
            self.get_object("passwordLabel").set_sensitive(button.get_active())
        elif button.get_name() == "enableProxyBypassButton":
            self.noProxyEntry.set_sensitive(button.get_active())
            self.noProxyEntry.grab_focus()

    def set_parent_window(self, window):
        self.networkConfigDialog.set_transient_for(window)



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2013 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import datetime
import logging

from rhsm import certificate

from subscription_manager import certlib
from subscription_manager import entcertlib
from subscription_manager import injection as inj

log = logging.getLogger(__name__)


class HealingActionInvoker(certlib.BaseActionInvoker):
    """
    An object used to run healing nightly. Checks cert validity for today, heals
    if necessary, then checks for 24 hours from now, so we theoretically will
    never have invalid certificats if subscriptions are available.

    NOTE: We may update entitlement status in this class, but we do not
          update entitlement certs, since we are inside a lock. So a
          EntCertActionInvoker.update() needs to follow a HealingActionInvoker.update()
    """

    def _do_update(self):
        action = HealingUpdateAction()
        return action.perform()


class HealingUpdateAction(object):
    """UpdateAction for ent cert healing.

    Core if entitlement certificate healing.

    Asks RHSM API to calculate entitlement status today, and tomorrow.
    If either show incomplete entitlement, ask the RHSM API to
    auto attach pools to fix entitlement.

    Attemps to avoid gaps in entitlement coverage.

    Used by rhsmd and subscription-manager if the "autoheal" options
    are enabled.

    Returns an EntCertUpdateReport with information about any ent
    certs that were changed.

    Plugin hooks:
        pre_auto_attach
        post_auto_attach
    """
    def __init__(self):
        self.cp_provider = inj.require(inj.CP_PROVIDER)
        self.uep = self.cp_provider.get_consumer_auth_cp()
        self.report = entcertlib.EntCertUpdateReport()
        self.plugin_manager = inj.require(inj.PLUGIN_MANAGER)

    def perform(self):
        # inject
        identity = inj.require(inj.IDENTITY)
        uuid = identity.uuid
        consumer = self.uep.getConsumer(uuid)

        if 'autoheal' not in consumer or not consumer['autoheal']:
            log.warn("Auto-heal disabled on server, skipping.")
            return 0

        try:

            today = datetime.datetime.now(certificate.GMT())
            tomorrow = today + datetime.timedelta(days=1)
            valid_today = False
            valid_tomorrow = False

            # Check if we're invalid today and heal if so. If we are
            # valid, see if 24h from now is greater than our "valid until"
            # date, and heal for tomorrow if so.

            cs = inj.require(inj.CERT_SORTER)

            cert_updater = entcertlib.EntCertActionInvoker()
            if not cs.is_valid():
                log.warn("Found invalid entitlements for today: %s" %
                        today)
                self.plugin_manager.run("pre_auto_attach", consumer_uuid=uuid)
                ents = self.uep.bind(uuid, today)
                self.plugin_manager.run("post_auto_attach", consumer_uuid=uuid,
                                        entitlement_data=ents)

                # NOTE: we need to call EntCertActionInvoker.update after Healing.update
                # otherwise, the locking get's crazy
                # hmm, we use RLock, maybe we could use it here
                self.report = cert_updater.update()
            else:
                valid_today = True

                if cs.compliant_until is None:
                    # Edge case here, not even sure this can happen as we
                    # should have a compliant until date if we're valid
                    # today, but just in case:
                    log.warn("Got valid status from server but no valid until date.")
                elif tomorrow > cs.compliant_until:
                    log.warn("Entitlements will be invalid by tomorrow: %s" %
                            tomorrow)
                    self.plugin_manager.run("pre_auto_attach", consumer_uuid=uuid)
                    ents = self.uep.bind(uuid, tomorrow)
                    self.plugin_manager.run("post_auto_attach", consumer_uuid=uuid,
                                            entitlement_data=ents)
                    self.report = cert_updater.update()
                else:
                    valid_tomorrow = True

            msg = "Entitlement auto healing was checked and entitlements"
            if valid_today:
                msg += " are valid today %s" % today
                if valid_tomorrow:
                    msg += " and tomorrow %s" % tomorrow
            log.debug(msg)

        except Exception as e:
            log.error("Error attempting to auto-heal:")
            log.exception(e)
            self.report._exceptions.append(e)
            return self.report
        else:
            log.debug("Auto-heal check complete.")
            return self.report



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2013 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#


class SubManPlugin(object):
    """Base class for all subscription-manager "rhsm-plugins"

    Plugins need to subclass SubManPlugin() to be found
    """
    name = None
    conf = None
    # if all_slots is set, the plugin will get registered to all slots
    # it is up to the plugin to handle providing callables
    all_slots = None

    # did we have hooks that match provided slots? aka
    # is this plugin going to be used
    found_slots_for_hooks = False

    def __init__(self, conf=None):
        if conf:
            self.conf = conf
        if self.conf is None:
            raise TypeError("SubManPlugin can not be constructed with conf=None")

    def __str__(self):
        return self.name or self.__class__.__name__

    @classmethod
    def get_plugin_key(cls):
        return ".".join([cls.__module__, cls.__name__])



from __future__ import print_function, division, absolute_import

# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import platform
import mock
from test.fixture import open_mock

from rhsmlib.facts import collector, firmware_info


class GetArchTest(unittest.TestCase):
    @mock.patch('platform.machine')
    def test_returns_arch(self, mock_machine):
        mock_machine.return_value = "hello_arch"
        arch = collector.get_arch()
        self.assertEqual("hello_arch", arch)

    def test_returns_arch_override(self):
        with open_mock(content="hello_arch"):
            arch = collector.get_arch(prefix="/does/not/exist")
            self.assertEqual("hello_arch", arch)

    def test_get_arch(self):
        self.assertEqual(platform.machine(), collector.get_arch())

    def test_get_platform_specific_info_provider(self):
        info_provider = firmware_info.get_firmware_collector(arch=platform.machine())
        self.assertTrue(info_provider is not None)



from __future__ import print_function, division, absolute_import

# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

from datetime import datetime, timedelta

import mock
from . import stubs

from rhsm import ourjson as json
from subscription_manager import action_client
from subscription_manager import content_action_client
from subscription_manager import entcertlib
from subscription_manager import identitycertlib
from subscription_manager import repolib
from subscription_manager import injection
import subscription_manager.injection as inj

from rhsmlib.facts import hwprobe

from rhsm.profile import RPMProfile
from rhsm.connection import GoneException
from rhsm.certificate import GMT

from .fixture import SubManFixture, set_up_mock_sp_store


CONSUMER_DATA = {'releaseVer': {'id': 1, 'releaseVer': '123123'},
                 'serviceLevel': "Pro Turbo HD Plus Ultra",
                 'owner': {'key': 'admin'},
                 'autoheal': 1,
                 'idCert': {'serial': {'serial': 3787455826750723380}}}


def mock_pkg_profile(packages):
    dict_list = []
    for pkg in packages:
        dict_list.append(pkg.to_dict())

    mock_file = mock.Mock()
    mock_file.read = mock.Mock(return_value=json.dumps(dict_list))

    mock_profile = RPMProfile(from_file=mock_file)
    return mock_profile


class ExceptionalException(Exception):
    pass


class ActionClientTestBase(SubManFixture):

    # on python 2.6+ we could set class decorators, but that doesn't
    # work on python2.4, so this...
    # http://www.voidspace.org.uk/python/mock/patch.html#patch-methods-start-and-stop
    def setUp(self):
        SubManFixture.setUp(self)
        # we have to have a reference to the patchers
        #self.patcher2 = mock.patch.object(entcertlib.EntCertUpdateAction, '_get_consumer_id')
        #self.entcertlib_updateaction_getconsumerid = self.patcher2.start()

        self.patcher3 = mock.patch.object(repolib.RepoUpdateActionCommand, 'perform')
        self.repolib_updateaction_perform = self.patcher3.start()

        self.patcher6 = mock.patch('subscription_manager.managerlib.persist_consumer_cert')
        self.managerlib_persist_consumer_cert = self.patcher6.start()

        # mock out all hardware fetching... we may need to fake socket counts
        self.hwprobe_getall_patcher = mock.patch.object(hwprobe.HardwareCollector, 'get_all')
        self.hwprobe_getall_mock = self.hwprobe_getall_patcher.start()
        self.hwprobe_getall_mock.return_value = {}

        self.patcher_entcertlib_writer = mock.patch("subscription_manager.entcertlib.Writer")
        self.entcertlib_writer = self.patcher_entcertlib_writer.start()

        self.patcher_entcertlib_action_syslogreport = mock.patch.object(entcertlib.EntCertUpdateAction, 'syslog_results')
        self.update_action_syslog_mock = self.patcher_entcertlib_action_syslogreport.start()

        # some stub certs
        stub_product = stubs.StubProduct('stub_product')
        self.stub_ent1 = stubs.StubEntitlementCertificate(stub_product)
        self.stub_ent2 = stubs.StubEntitlementCertificate(stub_product)
        self.stub_ent_expires_tomorrow = \
            stubs.StubEntitlementCertificate(stub_product,
                                             end_date=datetime.now() + timedelta(days=1))

        self.stub_ent_expires_tomorrow_ent_dir = \
            stubs.StubEntitlementDirectory([self.stub_ent_expires_tomorrow])

        self.local_ent_certs = [self.stub_ent1, self.stub_ent2]
        self.stub_entitled_proddir = \
            stubs.StubProductDirectory([stubs.StubProductCertificate(stub_product)])

        # local entitlement dir
        self.stub_ent_dir = stubs.StubEntitlementDirectory(self.local_ent_certs)
        inj.provide(inj.ENT_DIR, self.stub_ent_dir)

        self.mock_uep = mock.Mock()
        self.mock_uep.getCertificateSerials = mock.Mock(return_value=[{'serial': self.stub_ent1.serial},
                                                                        {'serial': self.stub_ent2.serial}])
        self.mock_uep.getConsumer = mock.Mock(return_value=CONSUMER_DATA)
        self.set_consumer_auth_cp(self.mock_uep)

        stub_release = {'releaseVer': '6.4'}
        self.mock_uep.getRelease = mock.Mock(return_value=stub_release)

        # we need to mock the consumers uuid with the mocked GoneExceptions
        # uuid
        self._inject_mock_valid_consumer(uuid="234234")

        self.repolib_updateaction_perform.return_value = 0

        # Setup a mock cert sorter to initiate the behaviour we want to test.
        # Must use a non-callable mock for our features dep injection
        # framework.
        self.mock_cert_sorter = mock.NonCallableMock()

        # TODO: need to provide return for "getRelease" for repolib stuff

        injection.provide(injection.CERT_SORTER, self.mock_cert_sorter)

        syspurpose_patch = mock.patch('subscription_manager.syspurposelib.SyncedStore')
        self.mock_sp_store = syspurpose_patch.start()
        self.mock_sp_store, self.mock_sp_store_contents = set_up_mock_sp_store(self.mock_sp_store)
        self.addCleanup(syspurpose_patch.stop)

    def tearDown(self):
        self.patcher3.stop()
        self.patcher6.stop()

        self.patcher_entcertlib_writer.stop()

        self.hwprobe_getall_patcher.stop()
        self.patcher_entcertlib_action_syslogreport.stop()


class TestContentActionClient(ActionClientTestBase):
    def test_init(self):
        actionclient = content_action_client.ContentActionClient()
        actionclient.update()


class TestActionClient(ActionClientTestBase):

    def test_init(self):
        actionclient = action_client.ActionClient()
        actionclient.update()

    # see bz #852706
    @mock.patch.object(entcertlib.EntCertActionInvoker, 'update')
    def test_gone_exception(self, mock_update):
        mock_update.side_effect = GoneException(410, "bye bye", " 234234")
        actionclient = action_client.ActionClient()
        self.assertRaises(GoneException, actionclient.update)

    # see bz #852706, except this time for idcertlib
    @mock.patch.object(identitycertlib.IdentityCertActionInvoker, 'update')
    def test_idcertlib_gone_exception(self, mock_update):
        mock_update.side_effect = GoneException(410, "bye bye", " 234234")
        actionclient = action_client.ActionClient()
        self.assertRaises(GoneException, actionclient.update)

        # just verify the certlib update worked
        report = actionclient.entcertlib.report
        self.assertTrue(self.stub_ent1.serial in report.valid)

    @mock.patch.object(entcertlib.EntCertActionInvoker, 'update')
    @mock.patch('subscription_manager.base_action_client.log')
    def test_entcertlib_update_exception(self, mock_log, mock_update):
        mock_update.side_effect = ExceptionalException()
        actionclient = action_client.ActionClient()
        actionclient.update()

        for call in mock_log.method_calls:
            if call[0] == 'exception' and isinstance(call[1][0], ExceptionalException):
                return
        self.fail("Did not ExceptionException in the logged exceptions")

    @mock.patch.object(identitycertlib.IdentityCertActionInvoker, 'update')
    @mock.patch('subscription_manager.base_action_client.log')
    def test_idcertlib_update_exception(self, mock_log, mock_update):
        mock_update.side_effect = ExceptionalException()
        actionclient = action_client.ActionClient()
        actionclient.update()

        for call in mock_log.method_calls:
            if call[0] == 'exception' and isinstance(call[1][0], ExceptionalException):
                return
        self.fail("Did not ExceptionException in the logged exceptions")

    def _stub_certificate_calls(self, stub_ents=None):
        stub_ents = stub_ents or []
        stub_ent_dir = stubs.StubEntitlementDirectory(stub_ents)

        inj.provide(inj.ENT_DIR, stub_ent_dir)

        # don't need to build real pem's, we mock out the writer anyway
        # so this just create a list of mock keys and stub ent certs
        stub_certificate_list = []
        for stub_cert in self.local_ent_certs:
            stub_certificate_list.append((mock.Mock(), stub_cert))

        # return a list of stub ent certs, could be new stubs, but
        # we already have created that
        self.mock_uep.getCertificates.return_value = stub_certificate_list

    # we need to simulate the client missing some ent certs
    @mock.patch.object(entcertlib.EntitlementCertBundleInstaller, 'build_cert')
    def test_missing(self, cert_build_mock):
        # mock no certs client side
        self._stub_certificate_calls()

        cert_build_mock.return_value = (mock.Mock(), self.stub_ent1)
        actionclient = action_client.ActionClient()
        actionclient.update()

        report = actionclient.entcertlib.report
        self.assertTrue(self.stub_ent1 in report.added)

    def test_rogue(self):
        # to mock "rogue" certs we need some local, that are not known to the
        # server so getCertificateSerials to return nothing
        self.mock_uep.getCertificateSerials = mock.Mock(return_value=[])
        self.set_consumer_auth_cp(self.mock_uep)
        actionclient = action_client.ActionClient()
        actionclient.update()

        report = actionclient.entcertlib.report
        # our local ent certs should be showing up as rogue
        self.assertTrue(self.local_ent_certs[0] in report.rogue)
        self.assertTrue(self.local_ent_certs[1] in report.rogue)

    @mock.patch.object(entcertlib.EntitlementCertBundleInstaller, 'build_cert')
    def test_expired(self, cert_build_mock):
        cert_build_mock.return_value = (mock.Mock(), self.stub_ent1)

        # this makes the stub_ent_dir report all ents as being expired
        # so we fetch new ones
        self.stub_ent_dir.list_expired = mock.Mock(
                return_value=self.stub_ent_dir.list())

        # we don't want to find replacements, so this forces a delete
        self.mock_uep.getCertificateSerials = mock.Mock(return_value=[])
        self.set_consumer_auth_cp(self.mock_uep)

        actionclient = action_client.ActionClient()
        actionclient.update()

        report = actionclient.entcertlib.report
        # the expired certs should be delete/rogue and expired
        #report = self.update_action_syslog_mock.call_args[0][0]
        self.assertTrue(self.stub_ent1 in report.rogue)

    @mock.patch.object(entcertlib.EntitlementCertBundleInstaller, 'build_cert')
    @mock.patch('subscription_manager.entcertlib.log')
    def test_exception_on_cert_write(self, mock_log, mock_cert_build):
        # this is basically the same as test_missing, expect we throw
        # an exception attempting to write the certs out
        self._stub_certificate_calls()

        mock_cert_build.side_effect = ExceptionalException()
        actionclient = action_client.ActionClient()
        # we should fail on the certlib.update, but keep going...
        # and handle it well.
        actionclient.update()

        for call in mock_log.method_calls:
            if call[0] == 'exception' and isinstance(call[1][0], ExceptionalException):
                return
        self.fail("Did not ExceptionException in the logged exceptions")


class TestHealingActionClient(TestActionClient):
    def test_healing_no_heal(self):
        self.mock_cert_sorter.is_valid = mock.Mock(return_value=True)
        self.mock_cert_sorter.compliant_until = datetime.now() + \
                timedelta(days=15)
        actionclient = action_client.HealingActionClient()
        actionclient.update(autoheal=True)
        self.assertFalse(self.mock_uep.bind.called)

    def test_healing_needs_heal(self):
        # need a stub product dir with prods with no entitlements,
        # don't have to mock here since we can actually pass in a product
        self.mock_cert_sorter.is_valid = mock.Mock(return_value=False)
        actionclient = action_client.HealingActionClient()
        actionclient.update(autoheal=True)
        self.assertTrue(self.mock_uep.bind.called)

    @mock.patch.object(entcertlib.EntitlementCertBundleInstaller, 'build_cert')
    def test_healing_needs_heal_tomorrow(self, cert_build_mock):
        # Valid today, but not valid 24h from now:
        self.mock_cert_sorter.is_valid = mock.Mock(return_value=True)
        self.mock_cert_sorter.compliant_until = datetime.now(GMT()) + \
                timedelta(hours=6)
        cert_build_mock.return_value = (mock.Mock(),
                self.stub_ent_expires_tomorrow)

        self._stub_certificate_calls([self.stub_ent_expires_tomorrow])
        actionclient = action_client.HealingActionClient()
        actionclient.update(autoheal=True)
        # see if we tried to update certs
        self.assertTrue(self.mock_uep.bind.called)

    # TODO: use Mock(wraps=) instead of hiding all logging
    @mock.patch('subscription_manager.healinglib.log')
    def test_healing_trigger_exception(self, mock_log):
        # Forcing is_valid to throw the type error we used to expect from
        # cert sorter using the product dir. Just making sure an unexpected
        # exception is logged and not bubbling up.
        self.mock_cert_sorter.is_valid = mock.Mock(side_effect=TypeError())
        actionclient = action_client.HealingActionClient()
        actionclient.update(autoheal=True)
        for call in mock_log.method_calls:
            if call[0] == 'exception' and isinstance(call[1][0], TypeError):
                return
        self.fail("Did not see TypeError in the logged exceptions")



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

from datetime import datetime
import unittest

from test.rhsm.unit import certdata
from rhsm.certificate import create_from_pem, CertificateException
from rhsm.certificate2 import Content, EntitlementCertificate, IdentityCertificate, Product, ProductCertificate

from mock import patch


class V1ProductCertTests(unittest.TestCase):
    def setUp(self):
        self.prod_cert_1 = create_from_pem(certdata.PRODUCT_CERT_V1_0)
        self.prod_cert_1.path = '/etc/pki/product/' + self.prod_cert_1.subject['CN'] + '.pem'
        self.prod_cert_2 = create_from_pem(certdata.PRODUCT_CERT_V1_1)
        self.prod_cert_2.path = '/etc/pki/product/' + self.prod_cert_2.subject['CN'] + '.pem'
        self.prod_cert_2_copy = create_from_pem(certdata.PRODUCT_CERT_V1_1)
        self.prod_cert_2_copy.path = '/etc/pki/product/' + self.prod_cert_2.subject['CN'] + '.pem'

    def test_factory_method_on_product_cert(self):
        self.assertEqual("1.0", str(self.prod_cert_1.version))
        self.assertTrue(isinstance(self.prod_cert_1, ProductCertificate))
        self.assertEqual(1, len(self.prod_cert_1.products))
        self.assertEqual('Awesome OS for x86_64 Bits',
                         self.prod_cert_1.products[0].name)
        self.assertEqual('100000000000002', self.prod_cert_1.subject['CN'])

    def test_os_old_cert(self):
        self.assertTrue(self.prod_cert_1.products[0].brand_type is None)

    def test_set_brand_type(self):
        brand_type = "OS"

        self.prod_cert_1.products[0].brand_type = brand_type
        self.assertEqual(brand_type, self.prod_cert_1.products[0].brand_type)

    def test_set_brand_name(self):
        brand_name = "Awesome OS Super"

        self.prod_cert_1.products[0].brand_name = brand_name
        self.assertEqual(brand_name, self.prod_cert_1.products[0].brand_name)

    def test_compare_certs(self):
        # Test comparison methods of Certificate object:
        # __lt__, __le__, __gt__, __ge__, __eq__, __ne__
        self.assertLess(self.prod_cert_1, self.prod_cert_2)
        self.assertLessEqual(self.prod_cert_1, self.prod_cert_2)
        self.assertGreater(self.prod_cert_2, self.prod_cert_1)
        self.assertGreaterEqual(self.prod_cert_2, self.prod_cert_1)
        self.assertEqual(self.prod_cert_1, self.prod_cert_1)
        self.assertNotEqual(self.prod_cert_1, self.prod_cert_2)

    def test_cert_membership(self):
        # Comparison methods are also used, when we test membership of certificate
        # in the list of certificates:
        list_of_certs = [self.prod_cert_1, self.prod_cert_2]
        self.assertIn(self.prod_cert_1, list_of_certs)
        self.assertIn(self.prod_cert_2_copy, list_of_certs)


class V1EntCertTests(unittest.TestCase):

    def setUp(self):
        self.ent_cert = create_from_pem(certdata.ENTITLEMENT_CERT_V1_0)

    def test_no_contents_throws_exception(self):
        self.assertRaises(CertificateException, create_from_pem, "")

    def test_junk_contents_throws_exception(self):
        self.assertRaises(CertificateException, create_from_pem,
                "DOESTHISLOOKLIKEACERTTOYOU?")

    def test_factory_method_on_ent_cert(self):
        self.assertEqual("1.0", str(self.ent_cert.version))
        self.assertTrue(isinstance(self.ent_cert, EntitlementCertificate))
        self.assertEqual(2012, self.ent_cert.start.year)
        self.assertEqual(2013, self.ent_cert.end.year)
        self.assertEqual("Awesome OS for x86_64", self.ent_cert.order.name)
        self.assertEqual(1, len(self.ent_cert.products))
        self.assertEqual('Awesome OS for x86_64 Bits',
                self.ent_cert.products[0].name)
        self.assertEqual('ff80808138574bd20138574d85a50b2f', self.ent_cert.subject['CN'])

    def test_is_valid(self):
        self.assertTrue(self.ent_cert.is_valid(on_date=datetime(2012, 12, 1)))
        self.assertFalse(self.ent_cert.is_valid(on_date=datetime(2014, 12, 1)))

    def test_order(self):
        self.assertEqual("Awesome OS for x86_64", self.ent_cert.order.name)

    def _find_content_by_label(self, content, label):
        """ Just pulls out content from a list if label matches. """
        for c in content:
            if c.label == label:
                return c

    def test_content(self):
        self.assertEqual(4, len(self.ent_cert.content))
        content = self._find_content_by_label(self.ent_cert.content,
                "always-enabled-content")
        self.assertEqual("always-enabled-content", content.name)
        self.assertEqual(True, content.enabled)
        self.assertEqual("/foo/path/always/$releasever", content.url)
        self.assertEqual("/foo/path/always/gpg", content.gpg)

    def test_access_path_tree_fails(self):
        # not supported for v3 certs
        self.assertRaises(AttributeError, getattr, self.ent_cert, '_path_tree')

    def test_check_path(self):
        # matches /foo/path/never
        self.assertTrue(self.ent_cert.check_path('/foo/path/never'))
        self.assertTrue(self.ent_cert.check_path('/foo/path/never/'))
        self.assertTrue(self.ent_cert.check_path('/foo/path/never/bar/a/b/c'))
        self.assertTrue(self.ent_cert.check_path('/foo/path/never/bar//a/b/c'))

    def test_check_path_with_var(self):
        # matches /path/to/$basearch/$releasever/awesomeos
        self.assertTrue(self.ent_cert.check_path('/path/to/foo/bar/awesomeos'))
        self.assertTrue(self.ent_cert.check_path('/path/to/foo/bar/awesomeos/'))
        self.assertTrue(self.ent_cert.check_path('/path/to/foo/bar/awesomeos/a/b/c'))

    def test_check_path_fail(self):
        self.assertFalse(self.ent_cert.check_path('foo'))
        self.assertFalse(self.ent_cert.check_path('/foo'))
        self.assertFalse(self.ent_cert.check_path('/foo/'))
        self.assertFalse(self.ent_cert.check_path('/foo/path/'))

    @patch('rhsm.certificate2.EntitlementCertificate._validate_v1_url')
    def test_download_url_identification(self, mock_validate):
        # there are 4 OIDs in the testing cert that should be checked, and
        # many others that should not. This verifies that exactly 4 OIDs get
        # checked.
        mock_validate.return_value = False
        self.ent_cert.check_path('/foo')
        self.assertEqual(mock_validate.call_count, 4)

    # TODO: test exception when cert major version is newer than we can handle


class V3CertTests(unittest.TestCase):

    def setUp(self):
        self.ent_cert = create_from_pem(certdata.ENTITLEMENT_CERT_V3_0)

    def test_factory_method_on_ent_cert(self):
        self.assertEqual("3.0", str(self.ent_cert.version))
        self.assertTrue(isinstance(self.ent_cert, EntitlementCertificate))
        self.assertEqual(2012, self.ent_cert.start.year)
        self.assertEqual(2013, self.ent_cert.end.year)

        self.assertEqual("Awesome OS for x86_64", self.ent_cert.order.name)

        self.assertEqual(1, len(self.ent_cert.products))
        self.assertEqual('Awesome OS for x86_64 Bits',
                self.ent_cert.products[0].name)
        self.assertEqual('ff80808139d9e26c0139da23489a0066',
                self.ent_cert.subject['CN'])

    def test_factory_method_without_ent_data(self):
        data = certdata.ENTITLEMENT_CERT_V3_0.split('-----BEGIN ENTITLEMENT DATA-----')[0]
        cert = create_from_pem(data)
        self.assertTrue(cert.content is None)
        self.assertTrue(cert.order is None)
        self.assertEqual(cert.products, [])

    def test_is_valid(self):
        self.assertTrue(self.ent_cert.is_valid(on_date=datetime(2012, 12, 1)))
        self.assertFalse(self.ent_cert.is_valid(on_date=datetime(2014, 12, 1)))

    def _find_content_by_label(self, content, label):
        """ Just pulls out content from a list if label matches. """
        for c in content:
            if c.label == label:
                return c

    def test_order(self):
        self.assertEqual(2, self.ent_cert.order.quantity_used)
        self.assertEqual("1", self.ent_cert.order.stacking_id)
        self.assertEqual("awesomeos-x86_64", self.ent_cert.order.sku)

    def test_content_enabled(self):
        self.assertEqual(4, len(self.ent_cert.content))
        content = self._find_content_by_label(self.ent_cert.content,
                "always-enabled-content")
        self.assertEqual("always-enabled-content", content.name)
        self.assertEqual(True, content.enabled)
        self.assertEqual(200, content.metadata_expire)
        self.assertEqual("/foo/path/always/$releasever", content.url)
        self.assertEqual("/foo/path/always/gpg", content.gpg)

    def test_content_disabled(self):
        self.assertEqual(4, len(self.ent_cert.content))
        content = self._find_content_by_label(self.ent_cert.content,
                "never-enabled-content")
        self.assertEqual("never-enabled-content", content.name)
        self.assertEqual(False, content.enabled)

    def test_content_no_arches(self):
        """Handle entitlement certs with no arch info on content"""
        content = self._find_content_by_label(self.ent_cert.content,
                                              "always-enabled-content")
        # we should get the default empty list if there are no arches
        # specified, ala the test entitlement cert
        self.assertEqual([], content.arches)

    @patch('os.unlink')
    def test_delete(self, unlink_mock):
        """ Entitlement cert deletion should cleanup key as well. """
        cert_path = "/etc/pki/entitlement/12345.pem"
        key_path = "/etc/pki/entitlement/12345-key.pem"
        self.ent_cert.path = cert_path
        self.unlinked = []
        self.ent_cert.delete()

        self.assertEqual(2, unlink_mock.call_count)
        self.assertEqual(cert_path, unlink_mock.call_args_list[0][0][0])
        self.assertEqual(key_path, unlink_mock.call_args_list[1][0][0])

    def test_cert_with_carriage_returns(self):
        # make sure it can parse a cert where the "-----" etc. lines end with
        # "\r\n" instead of just "\n". Failure to parse in this case was
        # discovered when trying to parse an employee-sku cert that jbowes
        # emailed to mhrivnak. the origin of the offending carriage returns is
        # unknown.
        crcert = certdata.ENTITLEMENT_CERT_V3_0.replace('-\n', '-\r\n')
        create_from_pem(crcert)

    def test_match_path(self):
        self.assertTrue(self.ent_cert.check_path('/path/to/awesomeos/x86_64'))
        self.assertTrue(self.ent_cert.check_path('/path/to/awesomeos//x86_64'))

    def test_match_deep_path(self):
        self.assertTrue(self.ent_cert.check_path('/path/to/awesomeos/x86_64/foo/bar'))

    def test_missing_pool(self):
        self.assertEqual(None, self.ent_cert.pool)


class V3_2CertTests(unittest.TestCase):

    def setUp(self):
        self.ent_cert = create_from_pem(certdata.ENTITLEMENT_CERT_V3_2)

    def test_read_pool(self):
        self.assertEqual("3.2", str(self.ent_cert.version))
        self.assertTrue(isinstance(self.ent_cert, EntitlementCertificate))
        self.assertEqual('8a8d01f53cda9dd0013cda9ed5100475',
                self.ent_cert.pool.id)


class TestEntCertV1KeyPath(unittest.TestCase):
    cert_data = certdata.ENTITLEMENT_CERT_V1_0

    def setUp(self):
        self.ent_cert = create_from_pem(self.cert_data)

    def test_key_path(self):
        self.ent_cert.path = "/etc/pki/entitlement/12345.pem"
        expected_key_path = "/etc/pki/entitlement/12345-key.pem"

        actual_key_path = self.ent_cert.key_path()
        self.assertEqual(actual_key_path, expected_key_path)

    def test_key_path_extra_pem(self):
        self.ent_cert.path = "/etc/pki/entitlement/12345-with-a.pem-in-the-name.pem"
        expected_key_path = "/etc/pki/entitlement/12345-with-a.pem-in-the-name-key.pem"

        actual_key_path = self.ent_cert.key_path()
        self.assertEqual(actual_key_path, expected_key_path)

    def test_key_path_no_pem(self):
        self.ent_cert.path = "/etc/pki/entitlement/12345"
        self.assertRaises(CertificateException, self.ent_cert.key_path)

    def test_key_path_no_slash(self):
        self.ent_cert.path = "12345.pem"
        expected_key_path = "12345-key.pem"

        actual_key_path = self.ent_cert.key_path()
        self.assertEqual(actual_key_path, expected_key_path)


class TestEntCertV3_0KeyPath(TestEntCertV1KeyPath):
    cert_data = certdata.ENTITLEMENT_CERT_V3_0


class TestEntCertV3_2KeyPath(TestEntCertV1KeyPath):
    cert_data = certdata.ENTITLEMENT_CERT_V3_2


class V3_2ContentArchCertTests(unittest.TestCase):

    def setUp(self):
        self.ent_cert = create_from_pem(certdata.ENTITLEMENT_CERT_V3_2_WITH_CONTENT_ARCH)

    def test_read_content_arches(self):
        for content in self.ent_cert.content:
            self.assertTrue(isinstance(content.arches, list))
            if content.label == 'always-enabled-content':
                self.assertEqual(['ALL'], content.arches)
            if content.label == 'awesomeos-x86_64-i386-content':
                self.assertTrue('i386' in content.arches)
                self.assertTrue('x86_64' in content.arches)
                self.assertFalse('ALL' in content.arches)


class IdentityCertTests(unittest.TestCase):

    def test_creation(self):
        id_cert = create_from_pem(certdata.IDENTITY_CERT)
        self.assertTrue(isinstance(id_cert, IdentityCertificate))
        self.assertEqual("URI:CN=redhat.local.rm-rf.ca", id_cert.alt_name)
        self.assertEqual("0f5d4617-d913-4a0f-be61-d8a9c88e1476", id_cert.subject['CN'])
        self.assertFalse(hasattr(id_cert, 'products'))

    def test_default_version(self):
        id_cert = create_from_pem(certdata.IDENTITY_CERT)
        self.assertTrue(isinstance(id_cert, IdentityCertificate))
        self.assertEqual('1.0', str(id_cert.version))


class ContentTests(unittest.TestCase):

    def test_enabled(self):
        c = Content(content_type="yum", name="mycontent", label="mycontent", enabled=None)
        self.assertTrue(c.enabled)
        c = Content(content_type="yum", name="mycontent", label="mycontent", enabled="1")
        self.assertTrue(c.enabled)
        c = Content(content_type="yum", name="mycontent", label="mycontent", enabled=True)
        self.assertTrue(c.enabled)
        c = Content(content_type="yum", name="mycontent", label="mycontent", enabled="0")
        self.assertFalse(c.enabled)
        self.assertRaises(CertificateException, Content, content_type="yum",
                name="mycontent", label="mycontent", enabled="5")

    def test_content_requires_type(self):
        self.assertRaises(CertificateException, Content, name="testcontent",
                          label="testcontent", enabled=True)
        self.assertRaises(CertificateException, Content, content_type=None,
                          name="testcontent", label="testcontent", enabled=True)
        self.assertRaises(CertificateException, Content, content_type="",
                          name="testcontent", label="testcontent", enabled=True)

    def test_arches_not_set(self):
        c = Content(content_type="yum", name="mycontent", label="mycontent", enabled=1)
        self.assertTrue(isinstance(c.arches, list))
        self.assertEqual([], c.arches)

    def test_arches_empty(self):
        c = Content(content_type="yum", name="mycontent", label="mycontent", enabled=1, arches=[])
        self.assertTrue(isinstance(c.arches, list))
        self.assertEqual([], c.arches)

    def test_arches(self):
        c = Content(content_type="yum", name="mycontent", label="mycontent", enabled=1, arches=['i386', 's390'])
        self.assertTrue(isinstance(c.arches, list))
        self.assertTrue('i386' in c.arches)
        self.assertTrue('s390' in c.arches)

    def test_arches_all(self):
        c = Content(content_type="yum", name="mycontent", label="mycontent", enabled=1, arches=['ALL'])
        self.assertTrue(isinstance(c.arches, list))
        self.assertTrue('ALL' in c.arches)

    def test_compare(self):
        c = Content(content_type="yum", name="mycontent", label="mycontent", enabled=1, arches=['ALL'])
        d = c
        e = Content(content_type="yum", name="mycontent", label="mycontent", enabled=1, arches=['ALL'])
        f = Content(content_type="yum", name="othercontent", label="othercontent", enabled=1)

        self.assertEqual(c, c)
        self.assertNotEqual(c, None)
        self.assertNotEqual(c, "not a content")
        self.assertEqual(c, d)
        self.assertEqual(c, e)
        self.assertNotEqual(c, f)


class ProductTests(unittest.TestCase):

    def test_arch_multi_valued(self):
        p = Product(id="pid", name="pname", architectures="i386,x86_64")
        self.assertEqual(2, len(p.architectures))
        self.assertEqual("i386", p.architectures[0])
        self.assertEqual("x86_64", p.architectures[1])

    def test_none_arch(self):
        p = Product(id="pid", name="pname")
        self.assertTrue(p.architectures is not None)
        self.assertTrue(isinstance(p.architectures, list))

    def test_no_brand_type(self):
        p = Product(id="pid", name="pname")
        self.assertTrue(p.brand_type is None)

    def test_brand_type(self):
        p = Product(id="pid", name="pname",
                    brand_type="pbrand_type")
        self.assertTrue(p.brand_type == "pbrand_type")

    def test_brand_type_none(self):
        p = Product(id="pid", name="pname",
                    brand_type=None)
        self.assertTrue(p.brand_type is None)

    def test_brand_type_empty_string(self):
        p = Product(id='pid', name='pname',
                    brand_type="")
        self.assertEqual(p.brand_type, "")

    def test_no_brand_name(self):
        p = Product(id="pid", name="pname")
        self.assertTrue(p.brand_name is None)

    def test_brand_name(self):
        p = Product(id="pid", name="pname",
                    brand_name="pbrand_name")
        self.assertTrue(p.brand_name == "pbrand_name")

    def test_brand_name_none(self):
        p = Product(id="pid", name="pname",
                    brand_name=None)
        self.assertTrue(p.brand_name is None)

    def test_brand_name_empty_string(self):
        p = Product(id="pid", name="pname",
                    brand_name="")
        self.assertEqual(p.brand_name, "")



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 - 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import errno
import os
import sys

from six import BytesIO
from zipfile import ZipFile, BadZipfile

from rhsm import certificate

from rct.commands import RCTCliCommand
from rct.printing import xstr
from subscription_manager.cli import InvalidCLIOptionError
from rhsm import ourjson as json

from subscription_manager.i18n import ugettext as _


def get_value(json_dict, path):
    current = json_dict
    for item in path.split("."):
        if item in current:
            current = current[item]
        else:
            return ""

    return current


class ZipExtractAll(ZipFile):
    """extend ZipFile with a safer extractall

    Zipfile() does not support extractall on python2.4, and the 2.6 versions
    are known to be unsafe in how they extract files. 2.6 version does not
    validate that files are within the archive root, or check that files are
    created safely.

    Contains helper methods for manipulating and reading
    the zipfile more easily in memory"""

    inner_zip = None

    def __init__(self, *args, **kwargs):
        """
        Validates the zip file
        """
        try:
            ZipFile.__init__(self, *args, **kwargs)
        except BadZipfile:
            print(_("Manifest zip is invalid."))
            sys.exit(1)

    def _get_inner_zip(self):
        if self.inner_zip is None:
            output = BytesIO(self.read(RCTManifestCommand.INNER_FILE))
            self.inner_zip = ZipExtractAll(output, 'r')
        return self.inner_zip

    def _read_file(self, file_path, is_inner=False):
        try:
            output = BytesIO(self.read(file_path))
            result = output.getvalue()
            output.close()
        except KeyError:
            try:
                if is_inner:
                    raise KeyError
                result = self._get_inner_zip()._read_file(file_path, True)
            except KeyError:
                raise Exception(_('Unable to find file "%s" in manifest.') % file_path)
        return result

    def _get_entitlements(self):
        results = []
        in_zip = self._get_inner_zip()
        for filename in in_zip.namelist():
            (read_path, read_file) = os.path.split(filename)
            if (read_path == os.path.join("export", "entitlements")) and (len(read_file) > 0):
                results.append(filename)
        return results

    def _open_excl(self, path):
        return os.fdopen(os.open(path, os.O_RDWR | os.O_CREAT | os.O_EXCL), 'wb')

    def _write_file(self, output_path, archive_path):
        outfile = self._open_excl(output_path)
        outfile.write(self.read(archive_path))
        outfile.close()

    def _is_secure(self, base, new_file):
        base_path = os.path.abspath(base)
        new_path = os.path.abspath(new_file)
        if not new_path.startswith(base_path):
            raise Exception(_('Manifest zip attempted to extract outside of the base directory.'))
        #traces symlink to source, and checks that it is valid
        real_new_path = os.path.realpath(new_path)
        if real_new_path != new_path:
            self._is_secure(base, real_new_path)
        elif os.path.islink(new_path):
            raise Exception(_('Unable to trace symbolic link.  Possibly circular linkage.'))

    def extractall(self, location, overwrite=False):
        self._is_secure(location, location)
        for path_name in self.namelist():
            (directory, filename) = os.path.split(path_name)
            directory = os.path.join(location, directory)
            self._is_secure(location, directory)
            if not os.path.exists(directory):
                os.makedirs(directory)
            new_location = os.path.join(directory, filename)
            self._is_secure(location, new_location)
            if (os.path.exists(new_location) and overwrite):
                os.remove(new_location)
            self._write_file(new_location, path_name)


class RCTManifestCommand(RCTCliCommand):

    INNER_FILE = "consumer_export.zip"

    def __init__(self, name="cli", aliases=None, shortdesc=None, primary=False):
        RCTCliCommand.__init__(self, name=name, aliases=aliases,
                shortdesc=shortdesc, primary=primary)

    def _get_usage(self):
        return _("%%prog %s [OPTIONS] MANIFEST_FILE") % self.name

    def _validate_options(self):
        manifest_file = self._get_file_from_args()
        if not manifest_file:
            raise InvalidCLIOptionError(_("You must specify a manifest file."))

        if not os.path.isfile(manifest_file):
            raise InvalidCLIOptionError(_("The specified manifest file does not exist."))

    def _extract_manifest(self, location, overwrite=False):
        # Extract the outer file
        archive = ZipExtractAll(self._get_file_from_args(), 'r')
        archive.extractall(location, overwrite)

        # now extract the inner file
        if location:
            inner_file = os.path.join(location, self.INNER_FILE)
        else:
            inner_file = self.INNER_FILE

        archive = ZipExtractAll(inner_file, 'r')
        archive.extractall(location, overwrite)

        # Delete the intermediate file
        os.remove(inner_file)


class CatManifestCommand(RCTManifestCommand):

    def __init__(self):
        RCTManifestCommand.__init__(self, name="cat-manifest", aliases=['cm'],
                               shortdesc=_("Print manifest information"),
                               primary=True)
        self.parser.add_option("--no-content", action="store_true",
                               default=False,
                               help=_("skip printing Content Sets"))

    def _print_section(self, title, items, indent=1, whitespace=True):
        # Allow a bit of customization of the tabbing
        pad = "\t" * (indent - 1)
        print(pad + title)
        pad += "\t"
        for item in items:
            if len(item) == 2:
                print("%s%s: %s" % (pad, item[0], xstr(item[1])))
            else:
                print("%s%s" % (pad, item[0]))
        if whitespace:
            print("")

    def _print_general(self, zip_archive):
        # Print out general data
        part = zip_archive._read_file(os.path.join("export", "meta.json"))
        data = json.loads(part)
        to_print = []
        to_print.append((_("Server"), get_value(data, "webAppPrefix")))
        to_print.append((_("Server Version"), get_value(data, "version")))
        to_print.append((_("Date Created"), get_value(data, "created")))
        to_print.append((_("Creator"), get_value(data, "principalName")))
        self._print_section(_("General:"), to_print)

    def _print_consumer(self, zip_archive):
        # Print out the consumer data
        part = zip_archive._read_file(os.path.join("export", "consumer.json"))
        data = json.loads(part)
        to_print = []
        to_print.append((_("Name"), get_value(data, "name")))
        to_print.append((_("UUID"), get_value(data, "uuid")))
        # contentAccessMode is entitlement if null, blank or non-present
        contentAccessMode = 'entitlement'
        if "contentAccessMode" in data and data["contentAccessMode"] == 'org_environment':
            contentAccessMode = 'Simple Content Access'
        to_print.append((_("Content Access Mode"), contentAccessMode))
        to_print.append((_("Type"), get_value(data, "type.label")))
        to_print.append((_("API URL"), get_value(data, "urlApi")))
        to_print.append((_("Web URL"), get_value(data, "urlWeb")))
        self._print_section(_("Consumer:"), to_print)

    def _get_product_attribute(self, name, data):
        return_value = None
        for attr in get_value(data, "pool.productAttributes"):
            if attr["name"] == name:
                return_value = attr["value"]
                break

        return return_value

    def _print_products(self, zip_archive):
        entitlements = zip_archive._get_entitlements()
        if len(entitlements) == 0:
            self._print_section(_("Subscriptions:"), [["None"]], 1, True)
            return

        for ent_file in entitlements:
            part = zip_archive._read_file(ent_file)
            data = json.loads(part)
            to_print = []
            to_print.append((_("Name"), get_value(data, "pool.productName")))
            to_print.append((_("Quantity"), get_value(data, "quantity")))
            to_print.append((_("Created"), get_value(data, "created")))
            to_print.append((_("Start Date"), get_value(data, "startDate")))
            to_print.append((_("End Date"), get_value(data, "endDate")))
            to_print.append((_("Service Level"), self._get_product_attribute("support_level", data)))
            to_print.append((_("Service Type"), self._get_product_attribute("support_type", data)))
            to_print.append((_("Architectures"), self._get_product_attribute("arch", data)))
            to_print.append((_("SKU"), get_value(data, "pool.productId")))
            to_print.append((_("Contract"), get_value(data, "pool.contractNumber")))
            to_print.append((_("Order"), get_value(data, "pool.orderNumber")))
            to_print.append((_("Account"), get_value(data, "pool.accountNumber")))
            virt_limit = self._get_product_attribute("virt_limit", data)
            to_print.append((_("Virt Limit"), virt_limit))
            require_virt_who = False
            if virt_limit:
                require_virt_who = True
            to_print.append((_("Requires Virt-who"), require_virt_who))

            entitlement_file = os.path.join("export", "entitlements", "%s.json" % data["id"])
            to_print.append((_("Entitlement File"), entitlement_file))
            #Get the certificate to get the version
            serial = data["certificates"][0]["serial"]["id"]

            cert_file = os.path.join("export", "entitlement_certificates", "%s.pem" % serial)
            to_print.append((_("Certificate File"), cert_file))

            try:
                cert = certificate.create_from_pem(zip_archive._read_file(cert_file).decode('utf-8'))
            except certificate.CertificateException as ce:
                raise certificate.CertificateException(
                        _("Unable to read certificate file '%s': %s") % (cert_file,
                        ce))
            to_print.append((_("Certificate Version"), cert.version))

            self._print_section(_("Subscription:"), to_print, 1, False)

            # Get the provided Products
            to_print = [(int(pp["productId"]), pp["productName"]) for pp in data["pool"]["providedProducts"]]

            self._print_section(_("Provided Products:"), sorted(to_print), 2, False)

            # Get the derived provided Products (if available)
            if "derivedProvidedProducts" in data["pool"]:
                to_print = [(int(pp["productId"]), pp["productName"]) for pp in data["pool"]["derivedProvidedProducts"]]
                self._print_section(_("Derived Products:"), sorted(to_print), 2, False)

            # Get the Content Sets
            if not self.options.no_content:
                to_print = [[item.url] for item in cert.content]
                self._print_section(_("Content Sets:"), sorted(to_print), 2, True)
            else:  # bz#1369577: print a blank line to separate subscriptions when --no-content in use
                print("")

    def _do_command(self):
        """
        Does the work that this command intends.
        """
        temp = ZipExtractAll(self._get_file_from_args(), 'r')
        # Print out the header
        print("\n+-------------------------------------------+")
        print(_("\tManifest"))
        print("+-------------------------------------------+\n")

        self._print_general(temp)
        self._print_consumer(temp)
        self._print_products(temp)


class DumpManifestCommand(RCTManifestCommand):

    def __init__(self):
        RCTManifestCommand.__init__(self, name="dump-manifest", aliases=['dm'],
                               shortdesc=_("Dump the contents of a manifest"),
                               primary=True)

        self.parser.add_option("--destination", dest="destination",
                               help=_("directory to extract the manifest to"))
        self.parser.add_option("-f", "--force", action="store_true",
                               dest="overwrite_files", default=False,
                               help=_("overwrite files which may exist"))

    def _extract(self, destination, overwrite):
        try:
            self._extract_manifest(destination, overwrite)
        except EnvironmentError as e:
            # IOError/OSError base class
            if e.errno == errno.EEXIST:
                # useful error for file already exists
                print(_('File "%s" exists. Use -f to force overwriting the file.') % e.filename)
            else:
                # generic error for everything else
                print(_("Manifest could not be written:"))
                print(e.strerror)
                if e.filename:
                    print(e.filename)
            return False
        return True

    def _do_command(self):
        """
        Does the work that this command intends.
        """
        if self.options.destination:
            if self._extract(self.options.destination, self.options.overwrite_files):
                print(_("The manifest has been dumped to the %s directory") % self.options.destination)
        else:
            if self._extract(os.getcwd(), self.options.overwrite_files):
                print(_("The manifest has been dumped to the current directory"))



#! /usr/bin/python

from __future__ import print_function, division, absolute_import

from subscription_manager.base_plugin import SubManPlugin

requires_api_version = "0.0"


class OldApiVersionPlugin(SubManPlugin):
    pass



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 - 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import base64
import os

from rhsm import certificate, _certificate
from rhsm.certificate2 import EntitlementCertificate

from rct.commands import RCTCliCommand
from rct.printing import printc, type_to_string

from subscription_manager.cli import InvalidCLIOptionError

from subscription_manager.i18n import ugettext as _


class RCTCertCommand(RCTCliCommand):

    def __init__(self, name="cli", aliases=None, shortdesc=None, primary=False):
        RCTCliCommand.__init__(self, name=name, aliases=aliases,
                shortdesc=shortdesc, primary=primary)

    def _get_usage(self):
        return _("%%prog %s [OPTIONS] CERT_FILE") % self.name

    def _create_cert(self):
        cert_file = self._get_file_from_args()
        try:
            return certificate.create_from_file(cert_file)
        except certificate.CertificateException as ce:
            raise InvalidCLIOptionError(
                    _("Unable to read certificate file '%s': %s") % (cert_file,
                        ce))

    def _validate_options(self):
        cert_file = self._get_file_from_args()
        if not cert_file:
            raise InvalidCLIOptionError(_("You must specify a certificate file."))

        if not os.path.isfile(cert_file):
            raise InvalidCLIOptionError(_("The specified certificate file does not exist."))


class CatCertCommand(RCTCertCommand):

    def __init__(self):
        RCTCliCommand.__init__(self, name="cat-cert", aliases=['cc'],
                               shortdesc=_("Print certificate information"),
                               primary=True)

        self.parser.add_option("--no-products", dest="no_products", action="store_true",
                               help=_("do not show the cert's product information"))
        self.parser.add_option("--no-content", dest="no_content", action="store_true",
                               help=_("do not show the cert's content info"))

    def _do_command(self):
        """
        Does the work that this command intends.
        """
        cert = self._create_cert()
        printc(cert, skip_content=self.options.no_content,
               skip_products=self.options.no_products)


class StatCertCommand(RCTCertCommand):

    def __init__(self):
        RCTCliCommand.__init__(self, name="stat-cert", aliases=['sc'],
                               shortdesc=_("Print certificate statistics and sizes"),
                               primary=True)

    def _do_command(self):
        cert = self._create_cert()
        pem = self._get_pem(self._get_file_from_args())
        print(_("Type: %s") % type_to_string(cert))
        print(_("Version: %s") % cert.version)
        print(_("DER size: %db") % self._get_der_size(pem))

        subject_key_id = self._get_subject_key_id(pem)
        if subject_key_id is not None:
            print(_("Subject Key ID size: %db") % len(subject_key_id))

        if isinstance(cert, EntitlementCertificate):
            content_len = 0
            if cert.content:
                content_len = len(cert.content)
            print(_("Content sets: %s") % content_len)

    def _get_pem(self, filename):
        return open(filename, 'r',).read()

    def _get_der_size(self, pem):
        parts = pem.split("-----BEGIN CERTIFICATE-----\n")
        cert = parts[1].split("-----END CERTIFICATE-----")[0]
        return len(base64.b64decode(cert))

    def _get_subject_key_id(self, pem):
        cert = _certificate.load(pem=pem)
        subject_key_id = cert.get_extension(oid="2.5.29.14")
        return subject_key_id



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2011 - 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of GPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#

import unittest

from rhsm.profile import Package, RPMProfile, get_profile, InvalidProfileType
from rhsm import ourjson as json
from mock import Mock

from nose.plugins.attrib import attr


@attr('functional')
class ProfileTests(unittest.TestCase):

    def test_get_rpm_profile(self):
        # This will fail if you're running tests on non-rpm based distros:
        profile = get_profile("rpm")
        pkg_dicts = profile.collect()
        self.assertEqual(len(profile.packages), len(pkg_dicts))

        # Everybody's gotta have at least 10 packages right?
        self.assertTrue(len(pkg_dicts) > 10)

        for pkg in pkg_dicts:
            self.assertTrue('name' in pkg)
            self.assertTrue('version' in pkg)
            self.assertTrue('version' in pkg)
            self.assertTrue('release' in pkg)
            self.assertTrue('epoch' in pkg)
            self.assertTrue('arch' in pkg)
            self.assertTrue('vendor' in pkg)

    def test_package_objects(self):
        profile = get_profile("rpm")
        for pkg in profile.packages:
            self.assertTrue(isinstance(pkg, Package))

    def test_get_profile_bad_type(self):
        self.assertRaises(InvalidProfileType, get_profile, "notreal")

    def test_load_profile_from_file(self):
        dummy_pkgs = [
                Package(name="package1", version="1.0.0", release=1, arch="x86_64"),
                Package(name="package2", version="2.0.0", release=2, arch="x86_64")]
        profile = self._mock_pkg_profile(dummy_pkgs)
        self.assertEqual(2, len(profile.packages))

    def _mock_pkg_profile(self, packages):
        """
        Turn a list of package objects into an RPMProfile object.
        """

        dict_list = []
        for pkg in packages:
            dict_list.append(pkg.to_dict())

        mock_file = Mock()
        mock_file.read = Mock(return_value=json.dumps(dict_list))

        mock_profile = RPMProfile(from_file=mock_file)
        return mock_profile

    def test_equality_different_object_type(self):
        dummy_pkgs = [
                Package(name="package1", version="1.0.0", release=1, arch="x86_64"),
                Package(name="package2", version="2.0.0", release=2, arch="x86_64")]
        profile = self._mock_pkg_profile(dummy_pkgs)
        self.assertFalse(profile == "hello")

    def test_equality_no_change(self):
        dummy_pkgs = [
                Package(name="package1", version="1.0.0", release=1, arch="x86_64"),
                Package(name="package2", version="2.0.0", release=2, arch="x86_64")]
        profile = self._mock_pkg_profile(dummy_pkgs)

        other = self._mock_pkg_profile(dummy_pkgs)
        self.assertTrue(profile == other)

    def test_equality_packages_added(self):
        dummy_pkgs = [
                Package(name="package1", version="1.0.0", release=1, arch="x86_64"),
                Package(name="package2", version="2.0.0", release=2, arch="x86_64")]
        profile = self._mock_pkg_profile(dummy_pkgs)

        dummy_pkgs.append(Package(name="package3", version="3.0.0", release=2,
            arch="x86_64"))
        other = self._mock_pkg_profile(dummy_pkgs)
        self.assertFalse(profile == other)

    def test_equality_packages_removed(self):
        dummy_pkgs = [
                Package(name="package1", version="1.0.0", release=1, arch="x86_64"),
                Package(name="package2", version="2.0.0", release=2, arch="x86_64")]
        profile = self._mock_pkg_profile(dummy_pkgs)

        dummy_pkgs.pop()
        other = self._mock_pkg_profile(dummy_pkgs)
        self.assertFalse(profile == other)

    def test_equality_packages_updated(self):
        dummy_pkgs = [
                Package(name="package1", version="1.0.0", release=1, arch="x86_64"),
                Package(name="package2", version="2.0.0", release=2, arch="x86_64")]
        profile = self._mock_pkg_profile(dummy_pkgs)

        # "Upgrade" package2:
        dummy_pkgs[1].version = "3.1.5"
        other = self._mock_pkg_profile(dummy_pkgs)
        self.assertFalse(profile == other)

    def test_equality_packages_replaced(self):
        dummy_pkgs = [
                Package(name="package1", version="1.0.0", release=1, arch="x86_64"),
                Package(name="package2", version="2.0.0", release=2, arch="x86_64")]
        profile = self._mock_pkg_profile(dummy_pkgs)

        # Remove package2, add package3:
        dummy_pkgs.pop()
        dummy_pkgs.append(Package(name="package3", version="3.0.0", release=2,
            arch="x86_64"))
        other = self._mock_pkg_profile(dummy_pkgs)
        self.assertFalse(profile == other)



from __future__ import print_function, division, absolute_import

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import datetime
from dateutil.tz import tzutc

from subscription_manager.gui import contract_selection
from nose.plugins.attrib import attr


def stubSelectedCallback(self, pool):
    pass


def stubCancelCallback(self):
    pass


@attr('gui')
class ContractSelection(unittest.TestCase):
    pool = {'productName': 'SomeProduct',
            'consumed': 3,
            'quantity': 10,
            'startDate': datetime.datetime.now(tz=tzutc()).isoformat(),
            'endDate': datetime.datetime.now(tz=tzutc()).isoformat(),
            'contractNumber': 'contractNumber',
            'attributes': [],
            'productAttributes': []}

    def test_contract_selection_show(self):
        cs = contract_selection.ContractSelectionWindow(selected_callback=stubSelectedCallback,
                                                        cancel_callback=stubCancelCallback)
        cs.show()

    def test_contract_selection_add_pool(self):
        cs = contract_selection.ContractSelectionWindow(selected_callback=stubSelectedCallback,
                                                         cancel_callback=stubCancelCallback)
        cs.add_pool(self.pool, 4)



from __future__ import print_function, division, absolute_import

# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

from subscription_manager.identity import ConsumerIdentity
import rhsm.connection as connection


class CPProvider(object):
    """
    CPProvider provides candlepin connections of varying authentication levels
    in order to avoid creating more than we need, and reuse the ones we have.

    Please try not to hold a self.cp or self.uep, instead the instance of CPProvider
    and use get_X_auth_cp() when a connection is needed.

    consumer_auth_cp: authenticates with consumer cert/key
    basic_auth_cp: also called admin_auth uses a username/password
    no_auth_cp: no authentication
    content_connection: ent cert based auth connection to cdn
    """

    consumer_auth_cp = None
    basic_auth_cp = None
    no_auth_cp = None
    content_connection = None

    # Initialize with default connection info from the config file
    def __init__(self):
        self.set_connection_info()
        self.correlation_id = None

    # Reread the config file and prefer arguments over config values
    # then recreate connections
    def set_connection_info(self,
                host=None,
                ssl_port=None,
                handler=None,
                cert_file=None,
                key_file=None,
                proxy_hostname_arg=None,
                proxy_port_arg=None,
                proxy_user_arg=None,
                proxy_password_arg=None,
                no_proxy_arg=None,
                restlib_class=connection.Restlib):
        self.cert_file = ConsumerIdentity.certpath()
        self.key_file = ConsumerIdentity.keypath()

        # only use what was passed in, let the lowest level deal with
        # no values and look elsewhere for them.
        self.server_hostname = host
        self.server_port = ssl_port
        self.server_prefix = handler
        self.proxy_hostname = proxy_hostname_arg
        self.proxy_port = proxy_port_arg
        self.proxy_user = proxy_user_arg
        self.proxy_password = proxy_password_arg
        self.no_proxy = no_proxy_arg
        self.restlib_class = restlib_class
        self.clean()

    # Set username and password used for basic_auth without
    # modifying previously set options
    def set_user_pass(self, username=None, password=None):
        self.username = username
        self.password = password
        self.basic_auth_cp = None

    # set up info for the connection to the cdn for finding release versions
    def set_content_connection_info(self, cdn_hostname=None, cdn_port=None):
        self.cdn_hostname = cdn_hostname
        self.cdn_port = cdn_port
        self.content_connection = None

    def set_correlation_id(self, correlation_id):
        self.correlation_id = correlation_id

    # Force connections to be re-initialized
    def clean(self):
        self.consumer_auth_cp = None
        self.basic_auth_cp = None
        self.no_auth_cp = None

    def get_consumer_auth_cp(self):
        if not self.consumer_auth_cp:
            self.consumer_auth_cp = connection.UEPConnection(
                    host=self.server_hostname,
                    ssl_port=self.server_port,
                    handler=self.server_prefix,
                    proxy_hostname=self.proxy_hostname,
                    proxy_port=self.proxy_port,
                    proxy_user=self.proxy_user,
                    proxy_password=self.proxy_password,
                    cert_file=self.cert_file, key_file=self.key_file,
                    correlation_id=self.correlation_id,
                    no_proxy=self.no_proxy,
                    restlib_class=self.restlib_class)
        return self.consumer_auth_cp

    def get_basic_auth_cp(self):
        if not self.basic_auth_cp:
            self.basic_auth_cp = connection.UEPConnection(
                    host=self.server_hostname,
                    ssl_port=self.server_port,
                    handler=self.server_prefix,
                    proxy_hostname=self.proxy_hostname,
                    proxy_port=self.proxy_port,
                    proxy_user=self.proxy_user,
                    proxy_password=self.proxy_password,
                    username=self.username,
                    password=self.password,
                    correlation_id=self.correlation_id,
                    no_proxy=self.no_proxy,
                    restlib_class=self.restlib_class)
        return self.basic_auth_cp

    def get_no_auth_cp(self):
        if not self.no_auth_cp:
            self.no_auth_cp = connection.UEPConnection(
                    host=self.server_hostname,
                    ssl_port=self.server_port,
                    handler=self.server_prefix,
                    proxy_hostname=self.proxy_hostname,
                    proxy_port=self.proxy_port,
                    proxy_user=self.proxy_user,
                    proxy_password=self.proxy_password,
                    correlation_id=self.correlation_id,
                    no_proxy=self.no_proxy,
                    restlib_class=self.restlib_class)
        return self.no_auth_cp

    def get_content_connection(self):
        if not self.content_connection:
            self.content_connection = connection.ContentConnection(host=self.cdn_hostname,
                                                                   ssl_port=self.cdn_port,
                                                                   proxy_hostname=self.proxy_hostname,
                                                                   proxy_port=self.proxy_port,
                                                                   proxy_user=self.proxy_user,
                                                                   proxy_password=self.proxy_password,
                                                                   no_proxy=self.no_proxy)
        return self.content_connection



from __future__ import print_function, division, absolute_import

# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging

from rhsmlib.facts import collector

log = logging.getLogger(__name__)


class CleanupCollector(collector.FactsCollector):
    no_uuid_platforms = ['powervm_lx86', 'xen-dom0', 'ibm_systemz']

    def get_all(self):
        cleanup_facts = {}
        dmi_socket_info = self.replace_socket_count_with_dmi()
        cleanup_facts.update(dmi_socket_info)
        return cleanup_facts

    def explain_lack_of_virt_uuid(self):
        # No virt.uuid equiv is available for guests on these hypervisors
        #virt_is_guest = self._collected_hw_info['virt.is_guest']
        if not self._is_a_virt_host_type_with_virt_uuids():
            log.debug("we don't sell virt uuids here")

    def _is_a_virt_host_type_with_virt_uuids(self):
        virt_host_type = self._collected_hw_info['virt.host_type']
        for no_uuid_platform in self.no_uuid_platforms:
            if virt_host_type.find(no_uuid_platform) > -1:
                return False
        return True

    def replace_socket_count_with_dmi(self):
        cleanup_info = {}
        # cpu topology reporting on xen dom0 machines is wrong. So
        # if we are a xen dom0, and we found socket info in dmiinfo,
        # replace our normal cpu socket calculation with the dmiinfo one
        # we have to do it after the virt data and cpu data collection
        if 'virt.host_type' not in self._collected_hw_info:
            return cleanup_info

        if not self._host_is_xen_dom0():
            return cleanup_info

        if 'dmi.meta.cpu_socket_count' not in self._collected_hw_info:
            return cleanup_info

        # Alright, lets munge up cpu socket info based on the dmi info.
        socket_count = int(self._collected_hw_info['dmi.meta.cpu_socket_count'])
        cleanup_info['cpu.cpu_socket(s)'] = socket_count

        if 'cpu.cpu(s)' not in self._collected_hw_info:
            return cleanup_info

        # And the cores per socket count as well
        dmi_cpu_cores_per_cpu = int(self._collected_hw_info['cpu.cpu(s)']) // socket_count
        cleanup_info['cpu.core(s)_per_socket'] = dmi_cpu_cores_per_cpu

        return cleanup_info

    def _host_is_xen_dom0(self):
        return self._collected_hw_info['virt.host_type'].find('dom0') > -1



from __future__ import print_function, division, absolute_import

import os
import logging
log = logging.getLogger("rhsm-app.unittests." + __name__)

from subscription_manager import cpuinfo

from . import fixture

test_path = os.path.dirname(__file__)
CPUINFOS = "data/cpuinfo"

cpu_data_dir = os.path.join(test_path, CPUINFOS)


class BaseCpuInfo(fixture.SubManFixture):
    expected = {}
    cpuinfo_class = None

    def _test(self, name):
        cpud = self._load_cpuinfo(name)
        exp = self.expected[name]

        x = self.cpuinfo_class.from_proc_cpuinfo_string(cpud)

        if 'cpu_count' in exp:
            self.assertEqual(exp['cpu_count'], len(x.cpu_info.processors))
        if 'model' in exp:
            self.assertEqual(exp['model'], x.cpu_info.model)
        if 'machine' in exp:
            self.assertEqual(exp['machine'], x.cpu_info.common['machine'])

    def _load_cpuinfo(self, name):
        f = open(os.path.join(cpu_data_dir, name), 'r')
        cpuinfo_data = f.read()
        return cpuinfo_data

    def _test_fact_key(self, key):
        bad_letters = ' .   '

        for bad_letter in bad_letters:
            self.assertFalse(bad_letter in key)


class TestX86_64CpuInfo(BaseCpuInfo):
    expected = {'x86_64-dell-e4310-1socket-2core-4cpu': {'cpu_count': 4,
                                      'model': '37'},
                'x86_64-dell-t7600-2sockets-6core-24cpu': {'cpu_count': 24,
                                            'model': '45'},
                'x86_64-4socket-8core-64cpu': {'cpu_count': 64},
                'armv7-samsung-1socket-2core-2cpu': {}}
    cpuinfo_class = cpuinfo.X86_64CpuInfo

    def test_x86_64_dell_e4310(self):
        self._test('x86_64-dell-e4310-1socket-2core-4cpu')

    def test_x86_64_64cpu(self):
        self._test('x86_64-4socket-8core-64cpu')

    def test_x86_64_24cpu_dell_t7600(self):
        self._test('x86_64-dell-t7600-2sockets-6core-24cpu')

    def test_armv7(self):
        # verify we fail nicely when given a completely different /proc/cpuinfo
        self._test('armv7-samsung-1socket-2core-2cpu')


class TestPpc64CpuInfo(BaseCpuInfo):
    cpuinfo_class = cpuinfo.Ppc64CpuInfo
    expected = {'ppc64-power8-16cpu-kvm':
                    {'cpu_count': 16,
                     'platform': 'pSeries',
                     'model': 'IBM pSeries (emulated by qemu)'},
                'ppc64-power8-160cpu-powernv':
                    {'cpu_count': 160,
                     'platform': 'powerNV',
                     'model': '8247-22L'}}

    def test_ppc64_power8_160cpu_powernv(self):
        self._test('ppc64-power8-160cpu-powernv')

    def test_ppc64_power8_16cpu_kvm(self):
        self._test('ppc64-power8-16cpu-kvm')


class TestPpc64leCpuInfo(BaseCpuInfo):
    cpuinfo_class = cpuinfo.Ppc64CpuInfo
    expected = {'ppc64le-power8-16cpu-lpar':
                {'cpu_count': 16,
                 'platform': 'pSeries',
                 'model': 'IBM,8247-22L',
                 'machine': 'CHRP IBM,8247-22L'}
              }

    def test_ppc64le_power8_16cpu_lpar(self):
        self._test('ppc64le-power8-16cpu-lpar')


class TestAarch64CpuInfo(BaseCpuInfo):
    cpuinfo_class = cpuinfo.Aarch64CpuInfo
    expected = {'aarch64-mustang-dev-rhel7-1socket-8core-8cpu':
                    {'cpu_count': 8,
                     'model_name': 'AArch64 Processor rev 0 (aarch64)'},
                'aarch64-hp-moonshot-1socket-8core-8cpu':
                    {'cpu_count': 8,
                     'model_name': None}}

    def test_mustang(self):
        self._test('aarch64-mustang-dev-rhel7-1socket-8core-8cpu')

    def test_moonshot(self):
        self._test('aarch64-hp-moonshot-1socket-8core-8cpu')



from __future__ import print_function, division, absolute_import

#
# branding - default localizable strings that can be overridden for app branding
#
# Copyright (c) 2011 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

"""
Overrideable i18n friendly module for branding specific strings. To override
values, drop a python module called <your name>_branding.py into this
directory. The module should contains a Branding class, whose instances have
attributes matching the names of those on DefaultBranding for any values you
want to override.
"""
import glob
import os
import sys

from subscription_manager.i18n import ugettext as _

__all__ = ["get_branding"]

_branding = None


def find_custom_branding():
    mod_path = os.path.dirname(__file__)
    mods = glob.glob(mod_path + "/*_branding.*")
    if len(mods) == 0:
        return None
    # we don't support multiple brandings
    branding_module = os.path.basename(mods[0]).split('.')[0]
    __import__(__name__ + "." + branding_module)
    mod = sys.modules[__name__ + "." + branding_module]
    return mod.Branding()


def get_branding():
    global _branding
    if _branding is None:
        custom_branding = find_custom_branding()
        _branding = Branding(custom_branding)
    return _branding


class Branding(object):

    def __init__(self, custom_branding=None):
        self._default = DefaultBranding()
        if custom_branding is None:
            custom_branding = EmptyBranding()
        self._custom = custom_branding

    def __getattr__(self, x):
        if hasattr(self._custom, x):
            return getattr(self._custom, x)
        else:
            return getattr(self._default, x)


class DefaultBranding(object):

    """
    Default branding. values are defined in init to help with i18n/l10n
    ordering.
    """

    def __init__(self):
        self.CLI_REGISTER = _("Register the system to the server")
        self.CLI_UNREGISTER = _("Unregister the system from the server")
        self.RHSMD_REGISTERED_TO_OTHER = \
                _("This system is registered to spacewalk")
        self.REGISTERED_TO_OTHER_WARNING = _("WARNING") + \
            "\n" + \
            _("You have already registered with spacewalk.")

        self.GUI_REGISTRATION_HEADER = \
                _("Please enter your account information:")
        self.GUI_FORGOT_LOGIN_TIP = \
                _("Contact your system administrator if you have forgotten your login or password")


class EmptyBranding(object):
    """
    Empty branding object to use in place of a custom branding
    (so we always fall back to the defaults.
    """
    pass
